-- IBM WBI Brokers Replay
-- Licensed Materials - Property of IBM Corp.
-- 5648-C63 (c) Copyright IBM Corp. 2006

-- Database Name: REPLAY

CREATE DATABASE REPLAY USING CODESET UTF-8 TERRITORY US COLLATE USING SYSTEM;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING APP_CTL_HEAP_SZ 512;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING APPGROUP_MEM_SZ 50000;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING APPLHEAPSZ      2048;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING CATALOGCACHE_SZ -1;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING CHNGPGS_THRESH  60;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING DBHEAP          1000;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING LOCKLIST        500;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING LOGBUFSZ        512;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING LOGFILSIZ       4096;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING LOGPRIMARY      50;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING LOGSECOND       25;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING MAXAPPLS        40;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING MAXLOCKS        22;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING MINCOMMIT       1;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING NUM_IOCLEANERS  1;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING NUM_IOSERVERS   3;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING PCKCACHESZ      2048;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING SOFTMAX         100;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING SORTHEAP        256;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING STMTHEAP        2048;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING DFT_DEGREE      1;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING DFT_PREFETCH_SZ 16;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING UTIL_HEAP_SZ    5000;
UPDATE DATABASE CONFIGURATION FOR REPLAY USING AVG_APPLS       4;

UPDATE DATABASE MANAGER CONFIGURATION USING SHEAPTHRES         10000;
UPDATE DATABASE MANAGER CONFIGURATION USING INTRA_PARALLEL     OFF;
UPDATE DATABASE MANAGER CONFIGURATION USING MAX_QUERYDEGREE    ANY;
UPDATE DATABASE MANAGER CONFIGURATION USING MAXAGENTS          400;
UPDATE DATABASE MANAGER CONFIGURATION USING NUM_POOLAGENTS     -1;
UPDATE DATABASE MANAGER CONFIGURATION USING NUM_INITAGENTS     0;
UPDATE DATABASE MANAGER CONFIGURATION USING FCM_NUM_BUFFERS    4096;
UPDATE DATABASE MANAGER CONFIGURATION USING PRIV_MEM_THRESH    20000;
UPDATE DATABASE MANAGER CONFIGURATION USING QUERY_HEAP_SZ      10000;

CONNECT TO REPLAY;

ALTER BUFFERPOOL IBMDEFAULTBP SIZE 200000;

-- Mimic tablespace

ALTER TABLESPACE SYSCATSPACE
      PREFETCHSIZE 8
      OVERHEAD 10.500000
      TRANSFERRATE 0.140000;

ALTER TABLESPACE TEMPSPACE1
      PREFETCHSIZE 256
      OVERHEAD 10.500000
      TRANSFERRATE 0.140000;

ALTER TABLESPACE USERSPACE1
      PREFETCHSIZE 32
      OVERHEAD 10.500000
      TRANSFERRATE 0.140000;



------------------------------------------------
-- DDL Statements for table "REPLAY  "."TOPIC_LIST"
------------------------------------------------
 
 CREATE TABLE "REPLAY  "."TOPIC_LIST"  (
		  "TOPIC_NAME" VARCHAR(500) NOT NULL , 
		  "TOPIC_ID" INTEGER NOT NULL )   
		 IN "USERSPACE1" NOT LOGGED INITIALLY ; 

-- DDL Statements for primary key on Table "REPLAY  "."TOPIC_LIST"

ALTER TABLE "REPLAY  "."TOPIC_LIST" 
	ADD CONSTRAINT "TOPICLIST_PRIMARY" PRIMARY KEY
		("TOPIC_ID");


-- DDL Statements for unique constraints on Table "REPLAY  "."TOPIC_LIST"

ALTER TABLE "REPLAY  "."TOPIC_LIST" 
	ADD CONSTRAINT "TOPIC_NAME_UNIQUE" UNIQUE
		("TOPIC_NAME");


------------------------------------------------
-- DDL Statements for table "REPLAY  "."MESSAGE_DATA"
------------------------------------------------
 
 CREATE TABLE "REPLAY  "."MESSAGE_DATA"  (
		  "SEQUENCE_NUMBER" INTEGER NOT NULL , 
		  "TOPIC_ID" INTEGER NOT NULL , 
		  "CONTENT" BLOB(32768) NOT LOGGED NOT COMPACT , 
		  "TIMESTAMP" TIMESTAMP NOT NULL )   
		 VALUE COMPRESSION  
		 IN "USERSPACE1" ; 




----------------------------

-- DDL Statements for Views

----------------------------
CREATE VIEW REPLAY.PRUNERVIEW(SEQUENCE_NUMBER,TOPIC_ID,TIMESTAMP) AS SELECT
SEQUENCE_NUMBER,TOPIC_ID,TIMESTAMP FROM REPLAY.MESSAGE_DATA;

CREATE VIEW REPLAY.LISTALLTOPICS(TOPIC) AS SELECT DISTINCT(TOPIC_NAME) from
REPLAY.TOPIC_LIST;

CREATE VIEW REPLAY.HIGHESTSEQUENCENUMBER(SEQUENCE_NUMBER,TOPIC_ID) AS SELECT
MAX(SEQUENCE_NUMBER),TOPIC_ID FROM REPLAY.MESSAGE_DATA GROUP BY TOPIC_ID;

CREATE VIEW REPLAY.MESSAGE_COUNT(TOTAL) AS SELECT COUNT('NULL') FROM REPLAY.MESSAGE_DATA;

-- DDL Statements for foreign keys on Table "REPLAY  "."MESSAGE_DATA"

ALTER TABLE "REPLAY  "."MESSAGE_DATA" 
	ADD CONSTRAINT "INSERT_NO_TOPICID" FOREIGN KEY
		("TOPIC_ID")
	REFERENCES "REPLAY  "."TOPIC_LIST"
		("TOPIC_ID")
	ON DELETE NO ACTION
	ON UPDATE NO ACTION
	ENFORCED
	ENABLE QUERY OPTIMIZATION;


-- All done, commit and restart for everything to take effect

COMMIT WORK;

CONNECT RESET;

DB2STOP;
DB2START;

TERMINATE;

