//=============================================================================
//
// IBM WBI Brokers Replay
//
// Licensed Materials - Property of IBM Corp.
// 5648-C63 (c) Copyright IBM Corp. 2004
//
//=============================================================================

package samples;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.regex.Pattern;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.Context;
import javax.naming.directory.InitialDirContext;

import com.ibm.broker.replay.api.ReplayJMSMessage;
import com.ibm.broker.replay.api.ReplaySignature;
import com.ibm.broker.replay.api.ReplayTopicSession;
import com.ibm.broker.replay.api.ReplayTopicSubscriber;
import com.ibm.mq.jms.JMSC;
import com.ibm.mq.jms.MQTopic;

public class ReplayClientCLI implements MessageListener
{
    private String topicConnectionFactoryName = "RTCF_IP_606";
    private String icf = "com.sun.jndi.fscontext.RefFSContextFactory";
    private String url = "file:.";

    private ReplayTopicSubscriber replaySub;
    private TopicSubscriber sub;

    private int numberOfPublications = 0;

    private long timeOfLastPublication = 0;

    private ReplaySignature sig = new ReplaySignature();

    private boolean isPerTopicData = true;

    private boolean isInteractive;
    private boolean isQuiet;

    private String topicName = "#";
    private SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy/MMM/dd_HH:mm:ss.S");

    private boolean shouldOutputToFile;
    private File outputFile;
    private PrintWriter pw;

    private boolean shouldPrintEveryMessage = true;

    private long firstPublicationTime = 0;
    private long stopTime = 0;
    private boolean isStopped = false;

    // Pattern used to remove the URL-like prefix that JMS prepends onto 
    // the real topic name.
    private static final Pattern topicPrefixPattern = Pattern.compile("topic://");

    private static final String FLAG1_tcfName                               = "-tcfName";                                   
    private static final String FLAG2_tcfName                               = "-tcf";                                   

    private static final String FLAG1_url                                   = "-url";                                   
    private static final String FLAG1_icf                                   = "-icf";                                   

    private static final String FLAG1_Topic                                 = "-Topic";                                 
    private static final String FLAG2_Topic                                 = "-T";
    private static final String FLAG1_StartNow                              = "-StartNow";                              
    private static final String FLAG2_StartNow                              = "-SN";
    private static final String FLAG1_StartTimestamp                        = "-StartTimestamp";                        
    private static final String FLAG2_StartTimestamp                        = "-ST";
    private static final String FLAG1_StartSequenceNumber                   = "-StartSequenceNumber";                   
    private static final String FLAG2_StartSequenceNumber                   = "-SSN";
    private static final String FLAG1_EndNever                              = "-EndNever";                              
    private static final String FLAG2_EndNever                              = "-En";
    private static final String FLAG1_EndTimestamp                          = "-EndTimestamp";                          
    private static final String FLAG2_EndTimestamp                          = "-ET";
    private static final String FLAG1_EndSequenceNumber                     = "-EndSequenceNumber";                     
    private static final String FLAG2_EndSequenceNumber                     = "-ESN";
    private static final String FLAG1_SamplingByTime                        = "-SamplingByTime";                        
    private static final String FLAG2_SamplingByTime                        = "-SBT";
    private static final String FLAG1_SamplingByNumberOfMessages            = "-SamplingByNumberOfMessages";            
    private static final String FLAG2_SamplingByNumberOfMessages            = "-SBNOM";
    private static final String FLAG1_PerTopicData                          = "-PerTopicData";                          
    private static final String FLAG2_PerTopicData                          = "-PTD";
    private static final String FLAG1_AggregatedTopicsData                  = "-AggregatedTopicsData";                  
    private static final String FLAG2_AggregatedTopicsData                  = "-ATD";
    private static final String FLAG3_AggregatedTopicsData                  = "-AggregatedTopicData";                   
    private static final String FLAG1_ASAPTiming                            = "-ASAPTiming";                            
    private static final String FLAG2_ASAPTiming                            = "-ASAP";
    private static final String FLAG1_OriginalTiming                        = "-OriginalTiming";                        
    private static final String FLAG2_OriginalTiming                        = "-OT";
    private static final String FLAG1_ThrottledTiming                       = "-ThrottledTiming";                       
    private static final String FLAG2_ThrottledTiming                       = "-TT";
    private static final String FLAG1_Interactive                           = "-Interactive";                           
    private static final String FLAG2_Interactive                           = "-i";
    private static final String FLAG1_Quiet                                 = "-Quiet";                         
    private static final String FLAG2_Quiet                                 = "-q";
    private static final String FLAG1_NoPrint                               = "-NoPrint";                                   
    private static final String FLAG2_NoPrint                               = "-NoP";
    private static final String FLAG1_Save                                  = "-Save";                                  
    private static final String FLAG2_Save                                  = "-S";
    private static final String FLAG1_Help                                  = "-help";                                  
    private static final String FLAG2_Help                                  = "-h";
    private static final String FLAG3_Help                                  = "--help";
    private static final String FLAG4_Help                                  = "-?";


    public static void main(String args[])
    {
        ReplayClientCLI replayClientCLI;
        try
        {
            replayClientCLI = new ReplayClientCLI();
            replayClientCLI.parseCommandLineArguments(args);            
            if(!replayClientCLI.isQuiet) replayClientCLI.printRequest();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            usage();
            System.exit(1);
            return; // this return to satisfy compiler - in case exit doesn't.
        }

        System.exit(replayClientCLI.run());
    }



    public ReplayClientCLI()
    {
    }



    private static void usage()
    {
        System.out.println("ReplayClientCLI Usage:"
                           //            1         2         3         4         5         6         7         8         9         10
                           //   1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
                           + "\njava samples.ReplayClientCLI [<options>...]"
                           + "\n  o Where subscription request options are:"
                           + "\n      " + FLAG1_Topic + " <topic>"
                           + "\n          * Sets the topic(s) to subscribe on.  Format as per the Broker."
                           + "\n            E.G. 'prices/FTSE100/ACME Co', 'prices/FTSE100/#',"
                           + "\n            'prices/+/ACME Co', 'prices/#'. Default is '#'.  Abbrv: " + FLAG2_Topic + "."
                           + "\n      " + FLAG1_StartNow + " | (" + FLAG1_StartTimestamp + " <timestamp>) | (" + FLAG1_StartSequenceNumber + " <sequence-number>)"
                           + "\n          * Sets the start point of the subscription, either from the current"
                           + "\n            time, an arbitrary time, or an arbitrary sequence number (NOTE: for"
                           + "\n            a single topic only).  Default is " + FLAG1_StartNow + ".  Abbrvs: " + FLAG2_StartNow + ", " + FLAG2_StartTimestamp + ","
                           + "\n            " + FLAG2_StartSequenceNumber + "."
                           + "\n      " + FLAG1_EndNever + " | (" + FLAG1_EndTimestamp + " <timestamp>) | (" + FLAG1_EndSequenceNumber + " <sequence-number>)"
                           + "\n          * Sets the end point of the subscription, either never end, until"
                           + "\n            an arbitrary time, or until an arbitrary sequence number (NOTE: for"
                           + "\n            a single topic only).  Default is " + FLAG1_EndNever + ".  Abbrvs: " + FLAG2_EndNever + ", " + FLAG2_EndTimestamp + ","
                           + "\n            " + FLAG2_EndSequenceNumber + "."
                           + "\n      " + FLAG1_SamplingByTime + " <period>) | " + FLAG1_SamplingByNumberOfMessages + " <sampling-interval-number>"
                           + "\n          * Turns on sampling.  Publications can be samples either by a"
                           + "\n            period or every Nth message.  Abbrvs: " + FLAG2_SamplingByTime + ", " + FLAG2_SamplingByNumberOfMessages + "."
                           + "\n      " + FLAG1_PerTopicData + " | " + FLAG1_AggregatedTopicsData + ""
                           + "\n          * Sets the sampling of messages to be done on either a per topic or"
                           + "\n            aggregated topics basis.  The default is " + FLAG1_PerTopicData + "."
                           + "\n            Abbrvs: " + FLAG2_PerTopicData + ", " + FLAG2_AggregatedTopicsData + "."
                           + "\n      " + FLAG1_ASAPTiming + " | " + FLAG1_OriginalTiming + " | (" + FLAG1_ThrottledTiming + " <period>)"
                           + "\n          * Sets the messages to be delivered either as soon as the Replay"
                           + "\n            Server can, with their original inter-message delays, or with a"
                           + "\n            delay of (at least) the period specified.  Default is " + FLAG1_ASAPTiming + "."
                           + "\n            Abbrvs: " + FLAG2_ASAPTiming + ", " + FLAG2_OriginalTiming + ", " + FLAG2_ThrottledTiming + "."
                           + "\n  o Where general options are:"
                           + "\n      " + FLAG1_Interactive + ""
                           + "\n          * Allows the user to type in commands to query the server or exit"
                           + "\n            the program.  By default this option is off, and the program will"
                           + "\n            exit after it has received the last message, as indicated"
                           + "\n            by " + FLAG1_EndTimestamp + " or " + FLAG1_EndSequenceNumber + ".  Abbrv: " + FLAG2_Interactive + "."
                           + "\n      " + FLAG1_Quiet + ""
                           + "\n          * Does not print progress information, only errors/warnings.  Does not affect the " + FLAG1_NoPrint + " flag."
                           + "\n            Abbrv: " + FLAG2_Quiet + "."
                           + "\n      " + FLAG1_NoPrint + ""
                           + "\n          * Do not print the string representation of every message, to STDOUT. Abbrv: " + FLAG2_NoPrint + "."
                           + "\n      " + FLAG1_Save + " <filename>"
                           + "\n          * Save the string representation of every message, to the outputFile. Abbrv: " + FLAG2_Save + "."
                           + "\n      " + FLAG1_tcfName + " <Topic-Connection-Factory-Name>"
                           + "\n          * Set the Topic Connection Factory name. Abbrv: " + FLAG2_tcfName + "."
                           + "\n      " + FLAG1_url + " <URL>"
                           + "\n          * Set the URL."
                           + "\n      " + FLAG1_icf + " <ICF>"
                           + "\n          * Set the ICF."
                           + "\n      " + FLAG1_Help + ""
                           + "\n          * Print this help text and exit. Abbrv: " + FLAG2_Help + "|" + FLAG3_Help + "|" + FLAG4_Help + "."
                           + "\n  o Where <timestamp> is either of the forms:"
                           + "\n      (1) 'NOW|START|YESTERDAY|TODAY|TOMORROW [[+|-|at] <period>]',"
                           + "\n          e.g. 'NOW', 'NOW - 1:00', 'NOW + 7:30', 'YESTERDAY at 9:30:00', "
                           + "\n          'TOMORROW at 15:59:59'."
                           + "\n      (2) time in milliseconds since January 1, 1970, 00:00:00 GMT: e.g. '1103203613628'."
                           + "\n  o Where <period> is of the form, '[[[H]H:}[m]m:][s...]s[.S[S...]]',"
                           + "\n      e.g. '15:07:24.125', '15:7:24.125', '7:24', '24.125', '24'."
                           + "\n  o All flags are case-insensitive (although topics are not, of course)."
                          );
    }

    private boolean flagMatch(String arg, String flag1)
    {
        return arg.equalsIgnoreCase(flag1);
    }

    private boolean flagMatch(String arg, String flag1, String flag2)
    {
        return 
        arg.equalsIgnoreCase(flag1)  ||  
        arg.equalsIgnoreCase(flag2);
    }

    private boolean flagMatch(String arg, String flag1, String flag2, String flag3)
    {
        return 
        arg.equalsIgnoreCase(flag1)  ||  
        arg.equalsIgnoreCase(flag2)  ||  
        arg.equalsIgnoreCase(flag3);
    }

    private boolean flagMatch(String arg, String flag1, String flag2, String flag3, String flag4)
    {
        return 
        arg.equalsIgnoreCase(flag1)  ||  
        arg.equalsIgnoreCase(flag2)  ||  
        arg.equalsIgnoreCase(flag3)  ||  
        arg.equalsIgnoreCase(flag4);
    }

    private void parseCommandLineArguments(String args[])
    {
        for(int i = 0; i < args.length; ++i)
        {
            String arg = args[i];

            if(flagMatch(arg, FLAG1_Topic, FLAG2_Topic))
            {
                setTopic(args[++i]);
            }
            else if(flagMatch(arg, FLAG1_StartNow, FLAG2_StartNow))
            {
                setStartNow();
            }
            else if(flagMatch(arg, FLAG1_StartTimestamp, FLAG2_StartTimestamp))
            {
                setStartTimestamp(TimeFormat.timestampParse(args[++i]));
            }
            else if(flagMatch(arg, FLAG1_StartSequenceNumber, FLAG2_StartSequenceNumber))
            {
                setStartSequenceNumber(new Long(args[++i]).longValue());
            }
            else if(flagMatch(arg, FLAG1_EndNever, FLAG2_EndNever))
            {
                setEndNever();
            }
            else if(flagMatch(arg, FLAG1_EndTimestamp, FLAG2_EndTimestamp))
            {
                setEndTimestamp(TimeFormat.timestampParse(args[++i]));
            }
            else if(flagMatch(arg, FLAG1_EndSequenceNumber, FLAG2_EndSequenceNumber))
            {
                setEndSequenceNumber(new Long(args[++i]).longValue());
            }
            else if(flagMatch(arg, FLAG1_SamplingByTime, FLAG2_SamplingByTime))
            {
                setSamplingByTime(TimeFormat.periodParse(args[++i]));
            }
            else if(flagMatch(arg, FLAG1_SamplingByNumberOfMessages, FLAG2_SamplingByNumberOfMessages))
            {
                setSamplingByNumberOfMessages(new Long(args[++i]).longValue());
            }
            else if(flagMatch(arg, FLAG1_PerTopicData, FLAG2_PerTopicData))
            {
                setPerTopicData();
            }
            else if(flagMatch(arg, FLAG1_AggregatedTopicsData, FLAG2_AggregatedTopicsData, FLAG3_AggregatedTopicsData))
            {
                setAggregatedTopicsData();
            }
            else if(flagMatch(arg, FLAG1_ASAPTiming, FLAG2_ASAPTiming))
            {
                sig.setTimingToAsap();
            }
            else if(flagMatch(arg, FLAG1_OriginalTiming, FLAG2_OriginalTiming))
            {
                sig.setTimingToOriginal();
            }
            else if(flagMatch(arg, FLAG1_ThrottledTiming, FLAG2_ThrottledTiming))
            {
                sig.setTimingToThrottled(TimeFormat.periodParse(args[++i]));
            }
            else if(flagMatch(arg, FLAG1_Save, FLAG2_Save))
            {
                String filename = args[++i];
                if(null == filename  ||  0 == filename.length())
                {
                    System.err.println("ReplayClientCLI: Must supply filename for option, '" + arg + "'.  Usage:-");
                    usage();
                    System.exit(1);
                }

                setSave(filename);
            }
            else if(flagMatch(arg, FLAG1_Interactive, FLAG2_Interactive))
            {
                setInteractive();
            }
            else if(flagMatch(arg, FLAG1_Quiet, FLAG2_Quiet))
            {
                setQuiet();
            }
            else if(flagMatch(arg, FLAG1_NoPrint, FLAG2_NoPrint))
            {
                setShouldNotPrintEveryMessage();
            }
            else if(flagMatch(arg, FLAG1_tcfName, FLAG2_tcfName))
            {
                String tcfName = args[++i];
                if(null == tcfName  ||  0 == tcfName.length())
                {
                    System.err.println("ReplayClientCLI: Must supply Topic Connection Factory Name for option, '" + arg + "'.  Usage:-");
                    usage();
                    System.exit(1);
                }

                setTCFName(tcfName);
            }
            else if(flagMatch(arg, FLAG1_url))
            {
                url = args[++i];
                if(null == url  ||  0 == url.length())
                {
                    System.err.println("ReplayClientCLI: Must supply URL for option, '" + arg + "'.  Usage:-");
                    usage();
                    System.exit(1);
                }
            }
            else if(flagMatch(arg, FLAG1_icf))
            {
                icf = args[++i];
                if(null == icf  ||  0 == icf.length())
                {
                    System.err.println("ReplayClientCLI: Must supply ICF for option, '" + arg + "'.  Usage:-");
                    usage();
                    System.exit(1);
                }
            }
            else if(flagMatch(arg, FLAG1_Help, FLAG2_Help, FLAG3_Help, FLAG4_Help))
            {
                usage();
                System.exit(0);
            }
            else
            {
                System.err.println("ReplayClientCLI: Unknown argument, '" + arg + "'.  Usage:-");
                usage();
                System.exit(1);
            }
        }
    }

    public void printRequest()
    {
        System.out.println("ReplayClientCLI: Requested Topic='" + topicName + "'");

        System.out.print("ReplayClientCLI: Requested Signature: ");

        if(sig.isStartNow())
        {
            System.out.print("StartNow");
        }
        else if(sig.isStartTimestamp())
        {
            System.out.print("StartTimestamp='" + TimeFormat.timestampFormat(sig.getStart()) + "'");
        }
        else
        {
            System.out.print("StartSequenceNumber=" + sig.getStart());
        }

        System.out.print(", ");

        if(sig.isEndNever())
        {
            System.out.print("EndNever");
        }
        else if(sig.isEndTimestamp())
        {
            System.out.print("EndTimestamp='" + TimeFormat.timestampFormat(sig.getEnd()) + "'");
        }
        else
        {
            System.out.print("EndSequenceNumber=" + sig.getEnd());
        }

        System.out.print(", ");

        if(sig.isSampled())
        {
            String on = (sig.isSampledOnAggregatedTopics() ? "On Aggregated Topics" : "On Topic");
            String by;
            String interval;

            if(sig.isSampledByTime())
            {
                by = "Sampled By Time";
                interval = String.valueOf(TimeFormat.periodFormat(sig.getSamplingInterval()));
            }
            else
            {
                by = "Sampled By Number Of Messages";
                interval = String.valueOf(sig.getSamplingInterval());
            }

            System.out.print(by + "(" + interval + ")" + on);
        }
        else
        {
            System.out.print("No Sampling");
        }

        System.out.print(", ");

        if(sig.isTimingAsap())
        {
            System.out.print("ASAP Timing");
        }
        else if(sig.isTimingOriginal())
        {
            System.out.print("Original Timing");
        }
        else if(sig.isTimingThrottled())
        {
            System.out.print("Throttled Timing (" + TimeFormat.periodFormat(sig.getThrottlingInterMessageTime()) + ")");
        }

        System.out.println(".");

        System.out.println("ReplayClientCLI: ReplaySignature: " + sig + ".");
    }

    public void setTopic(String string)
    {
        topicName = string;
    }

    public void setStartNow()
    {
        sig.setStartToNow();
    }

    public void setStartTimestamp(long l)
    {
        sig.setStartAsTimestamp(l);
    }

    public void setStartSequenceNumber(long l)
    {
        sig.setStartAsSequenceNumber(l);
    }

    public void setEndNever()
    {
        sig.setEndToNever();
    }

    public void setEndTimestamp(long l)
    {
        sig.setEndAsTimestamp(l);
    }

    public void setEndSequenceNumber(long l)
    {
        sig.setEndAsSequenceNumber(l);
    }

    public void setSamplingByTime(long l)
    {
        sig.setSamplingToByTime(l);
        if(isPerTopicData)
        {
            sig.setSamplingOnTopic();
        }
        else
        {
            sig.setSamplingOnAggregatedTopics();
        }
    }

    public void setSamplingByNumberOfMessages(long l)
    {
        sig.setSamplingToByNumberOfMessages(l);
        if(isPerTopicData)
        {
            sig.setSamplingOnTopic();
        }
        else
        {
            sig.setSamplingOnAggregatedTopics();
        }
    }

    public void setPerTopicData()
    {
        isPerTopicData = true;
        if(sig.isSampled())
        {
            sig.setSamplingOnTopic();
        }
    }

    public void setAggregatedTopicsData()
    {
        isPerTopicData = false;
        if(sig.isSampled())
        {
            sig.setSamplingOnAggregatedTopics();
        }
    }

    public void setSave(String filename)
    {
        shouldOutputToFile = true;
        outputFile = new File(filename);
        outputFile.delete();
        try
        {
            outputFile.createNewFile();
            pw = new PrintWriter(new FileWriter(outputFile, false));
        }
        catch(IOException e)
        {
            System.err.println("ReplayClientCLI: Cannot create new file, '" + filename + "'.");
            System.exit(1);
        }
    }

    public void setInteractive()
    {
        isInteractive = true;
    }

    public void setQuiet()
    {
        isQuiet = true;
    }

    public void setShouldPrintEveryMessage()
    {
        shouldPrintEveryMessage = true;
    }

    public void setShouldNotPrintEveryMessage()
    {
        shouldPrintEveryMessage = false;
    }

    public void setTCFName(String tcfName)
    {
        topicConnectionFactoryName = tcfName;
    }

    public int run()
    {
        int exitStatus = 1;

        try
        {
            Hashtable env = new Hashtable();
            env.put(Context.INITIAL_CONTEXT_FACTORY, icf);
            env.put(Context.PROVIDER_URL, url);
            env.put(Context.REFERRAL, "throw");

            Context ctx = new InitialDirContext(env);
            TopicConnectionFactory rtcf = (TopicConnectionFactory)ctx.lookup(topicConnectionFactoryName);
            ctx.close();

            try
            {
                TopicConnection conn = rtcf.createTopicConnection();

                try
                {
                    TopicSession sesh = conn.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);

                    Topic topic = sesh.createTopic(topicName);
                    ((MQTopic)topic).setBrokerVersion(JMSC.MQJMS_BROKER_V2);

                    try
                    {
                        sub = ((ReplayTopicSession)sesh).createReplaySubscriber(topic, null, sig);
                        replaySub = (ReplayTopicSubscriber)sub;

                        try
                        {
                            sub.setMessageListener(this);
                            conn.start();

                            if(isInteractive)
                            {
                                interaction();
                            }
                            else
                            {
                                waitForCompletion();
                            }

                            if(!isStopped)
                            {
                                if(!isQuiet) System.out.println("ReplayClientCLI: Closing ReplayTopicSubscriber...");
                                sub.close();
                                exitStatus = 0;
                            }
                        }
                        catch(JMSException e2)
                        {
                            System.err.println("ReplayClientCLI: Error.");
                            e2.printStackTrace();
                            exitStatus = 2;
                        }
                    }
                    catch(JMSException e1)
                    {
                        System.err.println("ReplayClientCLI: Error creating the replay subscriber.");
                        e1.printStackTrace();
                        exitStatus = 3;
                    }

                    if(!isQuiet) System.out.println("ReplayClientCLI: Closing TopicSession...");
                    sesh.close();
                }
                catch(JMSException e1)
                {
                    System.err.println("ReplayClientCLI: Error creating the TopicSession.");
                    e1.printStackTrace();
                    exitStatus = 4;
                }
                if(!isQuiet) System.out.println("ReplayClientCLI: Closing TopicConnection...");
                conn.close();
            }
            catch(JMSException e1)
            {
                System.err.println("ReplayClientCLI: Error creating the TopicSession.");
                e1.printStackTrace();
                exitStatus = 5;
            }
        }
        catch(Exception e)
        {
            System.err.println("ReplayClientCLI: Error.");
            e.printStackTrace();
            exitStatus = 6;
        }

        return exitStatus;
    }


    private void waitForCompletion()
    {
        final long RETRY_TIME = 5000;
        while(true)
        {
            // Wait for RETRY_TIME before testing for completion (since the request
            // may involve a round trip to the ReplayServer via the broker).
            while(true)
            {
                try
                {
                    Thread.sleep(RETRY_TIME);
                }
                catch(InterruptedException e)
                {
                    e.printStackTrace();
                }

                long elapsedTime = System.currentTimeMillis() - timeOfLastPublication;
                if(elapsedTime > RETRY_TIME)
                {
                    break;
                }
            }

            // Test for completion.
            try
            {
                if(replaySub.isReplayComplete())
                {
                    if(!isQuiet)
                    {
                        long timeSoFar = (System.currentTimeMillis() - firstPublicationTime);
                        long rate = (numberOfPublications * 1000) / timeSoFar;
                        String timingString  = "";
                        if (0 != numberOfPublications)
                        {
                            timingString  = " in time " + TimeFormat.periodFormat(timeSoFar) + " [" + rate + " pubs/s]";
                        }
                        System.out.println("ReplayClientCLI: Replay is complete (received " + numberOfPublications + " publications" + timingString + ").");
                    }
                    break;
                }
            }
            catch(JMSException e)
            {
                e.printStackTrace();
            }
        }
    }


    private void interaction()
    {
        BufferedReader commandLine = new java.io.BufferedReader(new InputStreamReader(System.in));

        System.out.println("ReplayClientCLI: Can enter one of: q=Query, s=Stop, x=eXit, pr=PrintRequest.");
        // keep reading stdin until the word "x" is typed
        while(true)
        {
            System.out.print("ReplayClientCLI: [q|s|x|pr] > ");
            String s;
            try
            {
                s = commandLine.readLine();
                if(s.equalsIgnoreCase("x"))
                {
                    break;
                }
                else if(s.equalsIgnoreCase("q"))
                {
                    System.out.println("ReplayClientCLI: Sending completion query...");
                    boolean isComplete = replaySub.isReplayComplete();
                    long timeSoFar;
                    if(isStopped)
                    {
                        timeSoFar = (stopTime - firstPublicationTime);
                    }
                    else
                    {
                        timeSoFar = (System.currentTimeMillis() - firstPublicationTime);
                    }
                    long rate = (numberOfPublications * 1000) / timeSoFar;
                    String timingString  = "";
                    if (0 != numberOfPublications)
                    {
                        timingString  = " in time " + TimeFormat.periodFormat(timeSoFar) + " [" + rate + " pubs/s]";
                    }
                    System.out.println("ReplayClientCLI: ...Replay is "+(isComplete?"":"in")+"complete (received " + numberOfPublications + " publications" + timingString + ").");
                }
                else if(s.equalsIgnoreCase("s"))
                {
                    if(isStopped)
                    {
                        System.out.println("ReplayClientCLI: Already stopped.");
                    }
                    else
                    {
                        System.out.println("ReplayClientCLI: Sending stop request...");
                        sub.close();
                        System.out.println("ReplayClientCLI: ...Stopped.");
                        stopTime = System.currentTimeMillis();
                        isStopped = true;
                    }
                }
                else if(s.equalsIgnoreCase("pr"))
                {
                    printRequest();
                }
                else if(!(0 == s.length()  ||  s.equals("")))
                {
                    System.out.println("ReplayClientCLI: Unknown command, '" + s + "'.");
                    System.out.println("ReplayClientCLI: Can enter one of: q=Query, s=Stop, x=eXit, pr=PrintRequest.");
                }
            }
            catch(IOException ioe)
            {
                ioe.printStackTrace();
            }
            catch(RuntimeException rte)
            {
                rte.printStackTrace();
            }
            catch(JMSException jmse)
            {
                jmse.printStackTrace();
            }
        }
    }


    public void onMessage(Message messageJMS)
    {
        if(0 == numberOfPublications)
        {
            firstPublicationTime = System.currentTimeMillis();
        }

        numberOfPublications++;
        timeOfLastPublication = System.currentTimeMillis();

        if(messageJMS == null  &&  shouldPrintEveryMessage)
        {
            System.err.println("ReplayClientCLI: ERROR: Received null message.  Message count=" + numberOfPublications + ".");
        }
        else
        {
            boolean error = false;

            boolean tracing = false;
            try
            {
                tracing = messageJMS.getBooleanProperty("com.ibm.broker.replay.track_publication");
            }
            catch(JMSException e1)
            {
            }

            ReplayJMSMessage replayJMSMessage = (ReplayJMSMessage)messageJMS;
            String errorText = "";

            String topicString = "<UNKNOWN-TOPIC>";
            try
            {
                // strip off the start of the topic name
                topicString = topicPrefixPattern.matcher(messageJMS.getJMSDestination().toString()).replaceFirst("");
            }
            catch(JMSException e1)
            {
                errorText += (error ? ", " : "") + "Cannot get Topic from ReplayJMSMessage";
                error = true;
            }

            String sequenceNumberString = "<UNKNOWN-SEQUENCE-NUMBER>";
            long sequenceNumber = 0;
            try
            {
                sequenceNumber = replayJMSMessage.getReplaySequenceNumber();
                sequenceNumberString = String.valueOf(sequenceNumber);
            }
            catch(JMSException e1)
            {
                errorText += (error ? ", " : "") + "Cannot get Sequence Number from ReplayJMSMessage";
                error = true;
            }

            String timestampString = "<UNKNOWN-TIME>";
            long timestamp = 0;
            try
            {
                timestamp = replayJMSMessage.getReplayTimestamp();
                timestampString = dateFormatter.format(new Date(timestamp));
            }
            catch(JMSException e1)
            {
                errorText += (error ? ", " : "") + "Cannot get Timestamp from ReplayJMSMessage";
                error = true;
            }


            String messageString;
            if(messageJMS instanceof TextMessage)
            {
                TextMessage textMessage = (TextMessage)messageJMS;
                try
                {
                    messageString = textMessage.getText();
                }
                catch(JMSException e)
                {
                    messageString = "<UNKNOWN>";
                    errorText += (error ? ", " : "") + "Cannot get Message text from JMS TextMessage";
                    error = true;
                }
            }
            else
            {
                messageString = messageJMS.toString();
            }

            if(shouldPrintEveryMessage)
            {
                if(false)
                {
                    System.out.println((isInteractive ? "\r" : "")
                                       + "Topic: '" + topicString + "', "
                                       + "SequenceNumber: " + sequenceNumberString + ", "
                                       + "Timestamp: " + timestamp + " (" + timestampString + "), "
                                       + "Message: '" + messageString + "'"
                                       + (tracing? ": TRACED MESSAGE." : ".")
                                      );
                }
                else if(true)
                {
                    System.out.println((isInteractive?"\r":"") + topicString + " #" + sequenceNumberString + " @" + timestamp + "(" + timestampString + "): " + messageString);
                }
                else if(false)
                {
                    System.out.println((isInteractive?"\r":"") + topicString + "#" + sequenceNumberString + "@" + timestamp + ":" + messageString);
                }
                else
                {
                    System.out.println((isInteractive?"\r":"") + topicString + "|" + sequenceNumberString + "|" + timestamp + "|" + messageString);
                }
            }

            if(shouldOutputToFile)
            {
                pw.println(messageString);
                pw.flush();
            }

            if(error)
            {
                System.err.println("ERROR: " + errorText + ".  "
                                   + "Received " + numberOfPublications + " publication" + (1==numberOfPublications?"":"s") + ", "
                                   + "@ " + TimeFormat.timestampFormat(System.currentTimeMillis()) + ".  "
                                   + "Publication:- "
                                   + "Timestamp: <" + timestampString + "> (" + TimeFormat.timestampFormat(timestamp) + "), "
                                   + "Topic: <" + topicString + ">, "
                                   + "SequenceNumber: <" + sequenceNumberString + ">, "
                                   + "Message: <" + messageString + ">.");
            }
        }
    }
}
