//=============================================================================
//
// IBM WBI Brokers Replay
//
// Licensed Materials - Property of IBM Corp.
// 5648-C63 (c) Copyright IBM Corp. 2004
//
//=============================================================================

package samples;

import javax.jms.*;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;

public class StockTickerPublisher implements Runnable
{
    Object key;
    String csvString;
    InputStreamReader inStream = null;
    BufferedReader buff = null;
    static StdInThread sts = null;

    private int replayDeletedRowCount = 0;
    private int replayCounter = 0;
    private int replayRowCount = 0;
    private double oldestRowCount = 0;

    private String contentString = null;
    private String symbol = null;
    private String companyName = null;
    private String companySymbol = null;
    static String stockExchange="";

    // Strings for default ICF and URL for JNDI connection
    static String icf = "com.sun.jndi.fscontext.RefFSContextFactory";
    static String url = "file:/C:/jndi";

    // Required JMS object declarations
    Context ctx = null;
    TopicConnectionFactory tcf = null;
    Topic topic = null;
    static TopicConnection conn = null;
    TopicSession session = null;
    TopicPublisher pub = null;

    public StockTickerPublisher(String args[])
    {
        output("PublisherStockTicker Application");

        try
        {
            // Create a Hashtable for JNDI required parameters
            Hashtable env = new Hashtable();
            env.put(Context.INITIAL_CONTEXT_FACTORY, icf);
            env.put(Context.PROVIDER_URL, url);
            env.put(Context.REFERRAL, "throw");

            // Initialise the JNDI Context
            ctx = new InitialDirContext(env);
        }
        catch(Exception e)
        {
            output("Unable to create JNDI context");
            e.printStackTrace();
            System.exit(0);
        }

        // Obtain the TCF from JNDI context
        try
        {
            tcf = (TopicConnectionFactory)ctx.lookup(args[0]);
        }
        catch(Exception e)
        {
            usage();
            e.printStackTrace();
            try
            {
                ctx.close();
            }
            catch(Exception ce)
            {
            }
            System.exit(0);
        }

        // Close our JNDI context
        try
        {
            ctx.close();
        }
        catch(Exception ce)
        {
        }

        // Create TopicConnection
        try
        {
            conn = tcf.createTopicConnection();
            output("Connection created");

            // Create TopicSession with required transactionality
            session = conn.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            output("Non-transacted Session created");
        }
        catch(JMSException e)
        {
            output("Unable to create JMS objects");
            e.printStackTrace();
            Exception le = e.getLinkedException();
            if(le != null)
                le.printStackTrace();

            // Close our JMS objects
            if(session != null)
                try
                {
                    session.close();
                }
                catch(JMSException je)
                {
                }
            if(conn != null)
                try
                {
                    conn.close();
                }
                catch(JMSException je)
                {
                }

            System.exit(0);
        }

        sts = new StdInThread();
        sts.start();
        Thread t = new Thread(this);
        t.start();
    }

    public void run()
    {
        while(sts.isGoing())
        {
            TreeMap companyNamesAndSymbols = new TreeMap();

            TreeMap FTSEcompanyNamesAndSymbols = new TreeMap();
            TreeMap DOWcompanyNamesAndSymbols = new TreeMap();
            TreeMap SPcompanyNamesAndSymbols = new TreeMap();

            if(stockExchange.equalsIgnoreCase("FTSE"))
            {
                companyNamesAndSymbols = createFTSECompanyNames(companyNamesAndSymbols);
                FTSEcompanyNamesAndSymbols = createFTSECompanyNames(FTSEcompanyNamesAndSymbols);
            }
            else if(stockExchange.equalsIgnoreCase("DOW"))
            {
                companyNamesAndSymbols = createDowJonesCompanyNames(companyNamesAndSymbols);
                DOWcompanyNamesAndSymbols = createDowJonesCompanyNames(DOWcompanyNamesAndSymbols);
            }
            else if(stockExchange.equalsIgnoreCase("SP"))
            {
                companyNamesAndSymbols = createSPCompanyNames(companyNamesAndSymbols);
                SPcompanyNamesAndSymbols =createSPCompanyNames(SPcompanyNamesAndSymbols);
            }
            else if(stockExchange.equalsIgnoreCase("ALL"))
            {
                companyNamesAndSymbols = createFTSECompanyNames(companyNamesAndSymbols);
                companyNamesAndSymbols = createDowJonesCompanyNames(companyNamesAndSymbols);
                companyNamesAndSymbols = createSPCompanyNames(companyNamesAndSymbols);

                FTSEcompanyNamesAndSymbols = createFTSECompanyNames(FTSEcompanyNamesAndSymbols);
                DOWcompanyNamesAndSymbols = createDowJonesCompanyNames(DOWcompanyNamesAndSymbols);
                SPcompanyNamesAndSymbols =createSPCompanyNames(SPcompanyNamesAndSymbols);

            }
            else
            {
                System.err.println("Unknown Stock exchange");   
                System.err.println("Please enter FTSE,DOW,SP,or ALL");
                System.exit(1);
            }


            Iterator it = companyNamesAndSymbols.keySet().iterator();

            while(it.hasNext() && sts.isGoing())
            {
                key = (String)it.next();

                companyName = (companyNamesAndSymbols.get(key)).toString();
                companySymbol = key.toString();

                URL netURL = null;
                URLConnection urlConn = null;
                try
                {
                    netURL = new URL("http://quote.yahoo.com/d/quotes.csv?s=" + key + "&f=kjsl1d1t1c1ohgvohm&e=.csv");

                    try
                    {
                        urlConn = netURL.openConnection();

                        try
                        {
                            inStream = new InputStreamReader(urlConn.getInputStream());

                            buff = new BufferedReader(inStream);

                            // get the quote as a csv string
                            try
                            {
                                csvString = buff.readLine();

                                String yearHigh = "?";
                                String yearLow = "?";
                                String ticker = "?";
                                String price = "?";
                                String tradeDate = "??";
                                String tradeTime = "??";
                                String opening = "?";
                                String daysLow = "?";
                                String daysHigh = "?";

                                // parse the csv string
                                StringTokenizer tokenizer = new StringTokenizer(csvString, ",");
                                try
                                {
                                    yearHigh = tokenizer.nextToken();
                                    yearLow = tokenizer.nextToken();
                                    ticker = tokenizer.nextToken();
                                    price = tokenizer.nextToken();
                                    tradeDate = tokenizer.nextToken();
                                    tradeTime = tokenizer.nextToken();
                                    opening = tokenizer.nextToken();
                                    daysLow = tokenizer.nextToken();
                                    daysHigh = tokenizer.nextToken();

                                    // Prepare to insert the tick into the database

                                    contentString =
                                    "Last price="
                                    + price
                                    + ",Last Trade="
                                    + tradeTime.substring(1, (tradeTime.length() - 1))
                                    + " on "
                                    + tradeDate.substring(1, (tradeDate.length() - 1))
                                    + ",Opening="
                                    + opening
                                    + ",Days Range="
                                    + daysLow
                                    + "-"
                                    + daysHigh
                                    + ",52wk Range = "
                                    + yearLow
                                    + "-"
                                    + yearHigh;

                                    publish("prices/" + companyName, contentString);
                                }
                                catch(RuntimeException e6)
                                {
                                    e6.printStackTrace();
                                }
                            }
                            catch(IOException e4)
                            {
                                e4.printStackTrace();
                            }
                        }
                        catch(IOException e3)
                        {
                            e3.printStackTrace();
                        }
                    }
                    catch(IOException e2)
                    {
                        e2.printStackTrace();
                    }
                }
                catch(MalformedURLException e1)
                {
                    e1.printStackTrace();
                }
            }
        }
        try
        {
            conn.close();
        }
        catch(JMSException e)
        {
            e.printStackTrace();
        }
    }

    private void publish(String topicName, String messageString)
    {
        // Create our JMS objects
        try
        {
            topic = session.createTopic(topicName);

            // Create TopicPublisher
            pub = session.createPublisher(topic);

        }
        catch(JMSException e)
        {
            output("Unable to create JMS objects");
            e.printStackTrace();
            Exception le = e.getLinkedException();
            if(le != null)
                le.printStackTrace();

            // Close our JMS objects
            if(pub != null)
                try
                {
                    pub.close();
                }
                catch(JMSException je)
                {
                }
            if(session != null)
                try
                {
                    session.close();
                }
                catch(JMSException je)
                {
                }
            if(conn != null)
                try
                {
                    conn.close();
                }
                catch(JMSException je)
                {
                }

            System.exit(0);
        }

        // Create and publish message.
        TextMessage msg;
        try
        {
            msg = session.createTextMessage();
            msg.setText(messageString);
            pub.publish(msg);
            output("Topic:'"+topicName+"'  Message published: '" + messageString + "'");

        }
        catch(JMSException e5)
        {
            e5.printStackTrace();
        }

        try
        {
            pub.close();
        }
        catch(JMSException je)
        {
            je.printStackTrace();
        }
    }

    private static void usage()
    {
        output("\n\n\n\n\n-------------------------------------------------------------------------------------------------");
        output("\tReplay StockTick Publisher");
        output("\n\tUsage\n");
        output("\t\tjava samples.StockTickerPublisher <TopicConnectionFactory> [jndiICF] [jndiURL] <FTSE|DOW|SP|ALL>\n");
        output("\tExample usage\n");
        output("\t\tjava samples.StockTickerPublisher IP_2004 com.sun.jndi.fscontext.RefFSContextFactory file:/c:/jndi FTSE");
        output("\t\tjava samples.StockTickerPublisher IP_2004 DOW");
        output("\n-------------------------------------------------------------------------------------------------");

    }

    private static void output(String s)
    {
        System.out.println(s);
    }

    private static TreeMap createFTSECompanyNames(TreeMap companyNamesAndSymbols)
    {
        companyNamesAndSymbols.put("^FTSE", "FSTE 100 INDEX");
        companyNamesAndSymbols.put("III.L", "3i GROUP");
        companyNamesAndSymbols.put("ANL.L", "ABBEY NATIONAL");
        companyNamesAndSymbols.put("AL.L", "ALLIANCE AND LEICS");
        companyNamesAndSymbols.put("AUN.L", "ALLIANCE UNICHEM");
        companyNamesAndSymbols.put("ALLD.L", "ALLIED DOMECQ");
        companyNamesAndSymbols.put("AVZ.L", "AMVESCAP");
        companyNamesAndSymbols.put("AAL.L", "ANGLO AMERICAN");
        companyNamesAndSymbols.put("ANTO.L", "ANTOFAGASTA");
        companyNamesAndSymbols.put("ABF.L", "ASSOC.BR.FOODS");
        companyNamesAndSymbols.put("AZN.L", "ASTRAZENECA");
        companyNamesAndSymbols.put("AV.L", "AVIVA");
        companyNamesAndSymbols.put("BSY.L", "B SKY B");
        companyNamesAndSymbols.put("BAA.L", "BAA");
        companyNamesAndSymbols.put("BA.L", "BAE SYSTEMS");
        companyNamesAndSymbols.put("BARC.L", "BARCLAYS");
        companyNamesAndSymbols.put("BG.L", "BG GROUP");
        companyNamesAndSymbols.put("BLT.L", "BHP BILLITON");
        companyNamesAndSymbols.put("BOC.L", "BOC GROUP");
        companyNamesAndSymbols.put("BOOT.L", "BOOTS GROUP");
        companyNamesAndSymbols.put("BP.L", "BP");
        companyNamesAndSymbols.put("BLND.L", "BR LAND CO");
        companyNamesAndSymbols.put("BB.L", "BRADFORD AND BING");
        companyNamesAndSymbols.put("BATS.L", "BRIT AM TOBACCO");
        companyNamesAndSymbols.put("BAY.L", "BRITISH AIRWAYS");
        companyNamesAndSymbols.put("BT.L", "BT GROUP");
        companyNamesAndSymbols.put("BNZL.L", "BUNZL PLC");
        companyNamesAndSymbols.put("CW.L", "CABLE & WIRELESS");
        companyNamesAndSymbols.put("CBRY.L", "CADBURY SCHWEPPE");
        companyNamesAndSymbols.put("CCL.L", "CARNIVAL");
        companyNamesAndSymbols.put("CNA.L", "CENTRICA");
        companyNamesAndSymbols.put("CPG.L", "COMPASS GROUP");
        companyNamesAndSymbols.put("DMGOa.L", "DAILY MAIL TST A");
        companyNamesAndSymbols.put("DGE.L,", "DIAGEO");
        companyNamesAndSymbols.put("DXNS.L", "DIXONS GROUP");
        companyNamesAndSymbols.put("EMA.L", "EMAP");
        companyNamesAndSymbols.put("ETI.L", "ENTERPRISE INNS");
        companyNamesAndSymbols.put("EXL.L", "EXEL");
        companyNamesAndSymbols.put("FP.L", "FRIENDS PROVDENT");
        companyNamesAndSymbols.put("GLH.L", "GALLAHER GROUP");
        companyNamesAndSymbols.put("GKN.L", "GKN");
        companyNamesAndSymbols.put("GSK.L", "GLAXOSMITHKLINE");
        companyNamesAndSymbols.put("GUS.L", "GUS");
        companyNamesAndSymbols.put("HNS.L", "HANSON");
        companyNamesAndSymbols.put("HAS.L", "HAYS");
        companyNamesAndSymbols.put("HBOS.L", "HBOS");
        companyNamesAndSymbols.put("HG.L", "HILTON GROUP");
        companyNamesAndSymbols.put("HSBA.L", "HSBC HOLDINGS");
        companyNamesAndSymbols.put("ICI.L", "ICI");
        companyNamesAndSymbols.put("IMT.L", "IMPERIAL TOBACCO");
        companyNamesAndSymbols.put("IHG.L", "INTERCONT HOTEL");
        companyNamesAndSymbols.put("ITV.L", "ITV");
        companyNamesAndSymbols.put("JMAT.L", "JOHNSON MATTHEY");
        companyNamesAndSymbols.put("KGF.L", "KINGFISHER");
        companyNamesAndSymbols.put("LAND.L", "LAND SECS GROUP");
        companyNamesAndSymbols.put("LGEN.L", "LEGAL AND GENERAL");
        companyNamesAndSymbols.put("LII.L", "LIBERTY INTL");
        companyNamesAndSymbols.put("LLOY.L", "LLOYDS TSB GRP");
        companyNamesAndSymbols.put("EMG.L", "MAN GROUP");
        companyNamesAndSymbols.put("OOM.L", "MMO2");
        companyNamesAndSymbols.put("MRW.L", "MORRISON SUPMKT");
        companyNamesAndSymbols.put("NGT.L", "NATIONAL GRID TR");
        companyNamesAndSymbols.put("NXT.L", "NEXT");
        companyNamesAndSymbols.put("NRK.L", "NORTHERN ROCK");
        companyNamesAndSymbols.put("OML.L", "OLD MUTUAL");
        companyNamesAndSymbols.put("PSON.L", "PEARSON");
        companyNamesAndSymbols.put("PRU.L", "PRUDENTIAL");
        companyNamesAndSymbols.put("RB.L", "RECKITT BENCKSR");
        companyNamesAndSymbols.put("REL.L", "REED ELSEVIER");
        companyNamesAndSymbols.put("RTO.L", "RENTOKIL INITIAL");
        companyNamesAndSymbols.put("RTR.L", "REUTERS GROUP");
        companyNamesAndSymbols.put("REX.L", "REXAM PLC");
        companyNamesAndSymbols.put("RIO.L", "RIO TINTO");
        companyNamesAndSymbols.put("RR.L", "ROLLS ROYCE GP");
        companyNamesAndSymbols.put("RSA.L", "ROYAL & SUN ALL");
        companyNamesAndSymbols.put("RBS.L", "ROYAL BANK SCOT");
        companyNamesAndSymbols.put("SAB.L", "SABMILLER");
        companyNamesAndSymbols.put("SGE.L", "SAGE GROUP");
        companyNamesAndSymbols.put("SBRY.L", "SAINSBURY (J)");
        companyNamesAndSymbols.put("SDR.L", "SCHRODERS");
        companyNamesAndSymbols.put("SDRt.L", "SCHRODERS NV");
        companyNamesAndSymbols.put("SCTN.L", "SCOT AND NEWCASTLE");
        companyNamesAndSymbols.put("SSE.L", "SCOT AND STH ENRGY");
        companyNamesAndSymbols.put("SPW.L", "SCOTTISH POWER");
        companyNamesAndSymbols.put("SVT.L", "SEVERN TRENT");
        companyNamesAndSymbols.put("SHEL.L", "SHELL TRNPT(REG)");
        companyNamesAndSymbols.put("SHP.L", "SHIRE PHARM GRP");
        companyNamesAndSymbols.put("SN.L", "SMITH AND NEPHEW");
        companyNamesAndSymbols.put("SMIN.L", "SMITHS GROUP");
        companyNamesAndSymbols.put("STAN.L", "STNDRD CHART BK");
        companyNamesAndSymbols.put("TSCO.L", "TESCO");
        companyNamesAndSymbols.put("TOMK.L", "TOMKINS");
        companyNamesAndSymbols.put("ULVR.L", "UNILEVER");
        companyNamesAndSymbols.put("UU.L", "UNITED UTILITIES");
        companyNamesAndSymbols.put("UUa.L", "UNITED UTILTS A");
        companyNamesAndSymbols.put("VOD.L", "VODAFONE GROUP");
        companyNamesAndSymbols.put("WTB.L", "WHITBREAD");
        companyNamesAndSymbols.put("WMH.L", "WILLIAM HILL");
        companyNamesAndSymbols.put("WOS.L", "WOLSELEY");
        companyNamesAndSymbols.put("WPP.L", "WPP GROUP");
        companyNamesAndSymbols.put("XTA.L", "XSTRATA");
        companyNamesAndSymbols.put("YELL.L", "YELL GROUP");

        return companyNamesAndSymbols;
    }

    private static TreeMap createDowJonesCompanyNames(TreeMap companyNamesAndSymbols)
    {
        companyNamesAndSymbols.put("^DJI", "DJ INDU AVERAGE");
        companyNamesAndSymbols.put("AA", "ALCOA Inc ");
        companyNamesAndSymbols.put("AIG","AMERICAN INTERNATION GROUP,INC");
        companyNamesAndSymbols.put("AXP","AMERICAN EXPRESS CO");
        companyNamesAndSymbols.put("BA","BOEING CO");
        companyNamesAndSymbols.put("C","CITIGROUP");
        companyNamesAndSymbols.put("CAT","CATERPILLAR INC");
        companyNamesAndSymbols.put("DD","DU PONT CO");
        companyNamesAndSymbols.put("DIS","WALT DISNEY CO");
        companyNamesAndSymbols.put("GE","GENERAL ELEC CO");
        companyNamesAndSymbols.put("GM","GENERAL MOTORS");
        companyNamesAndSymbols.put("HD","HOME DEPOT INC");
        companyNamesAndSymbols.put("HON","HONEYWELL INTL");
        companyNamesAndSymbols.put("HPQ","HEWLETT-PACKARD");
        companyNamesAndSymbols.put("IBM","INTL BUS MACHINE");
        companyNamesAndSymbols.put("INTC","INTEL CORP");
        companyNamesAndSymbols.put("JNJ","JOHNSON&JOHNSON");
        companyNamesAndSymbols.put("JPM","JPMORGAN CHASE");
        companyNamesAndSymbols.put("KO","COCA-COLA CO");
        companyNamesAndSymbols.put("MCD","MCDONALDS CORP");
        companyNamesAndSymbols.put("MMM","3M COMPANY");
        companyNamesAndSymbols.put("MO","ALTRIA GROUP");
        companyNamesAndSymbols.put("MRK","MERCK & CO");
        companyNamesAndSymbols.put("MSFT","MICROSOFT CP");
        companyNamesAndSymbols.put("PFE","PFIZER INC");
        companyNamesAndSymbols.put("PG","PROCTER & GAMBLE");
        companyNamesAndSymbols.put("SBC","SBC COMMS");
        companyNamesAndSymbols.put("UTX","UNITED TECH CP");
        companyNamesAndSymbols.put("VZ","VERIZON COMMS");
        companyNamesAndSymbols.put("WMT","WAL-MART STORES");
        companyNamesAndSymbols.put("XOM","EXXON MOBILE");

        return companyNamesAndSymbols;
    }

    private static TreeMap createSPCompanyNames(TreeMap companyNamesAndSymbols)
    {
        companyNamesAndSymbols.put("A","AGILENT TECH");
        companyNamesAndSymbols.put("AA","ALCOA INC");
        companyNamesAndSymbols.put("AAPL","APPLE COMP INC");
        companyNamesAndSymbols.put("ABC","AMERISOURCEBERGN");
        companyNamesAndSymbols.put("ABI","APPLIED BIOSYS");
        companyNamesAndSymbols.put("ABK","AMBAC FINL");
        companyNamesAndSymbols.put("ABS","ALBERTSONS INC");
        companyNamesAndSymbols.put("ABT","ABBOTT LABS");
        companyNamesAndSymbols.put("ACE","ACE LTD");
        companyNamesAndSymbols.put("ACS","AFFL COMPUTER");
        companyNamesAndSymbols.put("ACV","ALBERTO CULVER");
        companyNamesAndSymbols.put("ADBE","ADOBE SYS");
        companyNamesAndSymbols.put("ADCT","ADC TELECOMM"); 
        companyNamesAndSymbols.put("ADI","ANALOG DEVICES");
        companyNamesAndSymbols.put("ADM","ARCHER-DANIELS");
        companyNamesAndSymbols.put("ADP","AUTOMATIC DATA");
        companyNamesAndSymbols.put("ADSK","AUTODESK INC");
        companyNamesAndSymbols.put("AEE","AMEREN CORP"); 
        companyNamesAndSymbols.put("AEP","AMER ELEC PWR"); 
        companyNamesAndSymbols.put("AES","THE AES CORP");
        companyNamesAndSymbols.put("AET","AETNA INC");
        companyNamesAndSymbols.put("AFL","AFLAC INC");
        companyNamesAndSymbols.put("AGN","ALLERGAN INC"); 
        companyNamesAndSymbols.put("AHC","AMERADA HESS");
        companyNamesAndSymbols.put("AIG","AMER INTL GROUP");  
        companyNamesAndSymbols.put("AIV","APT INV MANAGE"); 
        companyNamesAndSymbols.put("ALL","ALLSTATE CP");
        companyNamesAndSymbols.put("ALTR","ALTERA CORP");
        companyNamesAndSymbols.put("AMAT","APPLIED MATL");
        companyNamesAndSymbols.put("AMCC","APPLD MICRO");
        companyNamesAndSymbols.put("AMD","ADV MICRO DEVICE"); 
        companyNamesAndSymbols.put("AMGN","AMGEN");
        companyNamesAndSymbols.put("AN","AUTONATION INC"); 
        companyNamesAndSymbols.put("ANDW","ANDREW CORP");
        companyNamesAndSymbols.put("AOC","AON CORP");
        companyNamesAndSymbols.put("APA","APACHE CORP"); 
        companyNamesAndSymbols.put("APC","ANADARKO PETE"); 
        companyNamesAndSymbols.put("APCC","AMER POWR CONV");
        companyNamesAndSymbols.put("APD","AIR PRODS & CHEM");
        companyNamesAndSymbols.put("APOL","APOLLO GROUP");
        companyNamesAndSymbols.put("ASD","AMER STANDARD");
        companyNamesAndSymbols.put("ASH","ASHLAND INC");
        companyNamesAndSymbols.put("ASO","AMSOUTH BANCORP"); 
        companyNamesAndSymbols.put("AT","ALLTEL CORP");
        companyNamesAndSymbols.put("ATH","ANTHEM INC");
        companyNamesAndSymbols.put("ATI","ALLEGHENY TECH");
        companyNamesAndSymbols.put("AV","AVAYA INC");
        companyNamesAndSymbols.put("AVP","AVON PRODUCTS"); 
        companyNamesAndSymbols.put("AVY","AVERY DENNISON");
        companyNamesAndSymbols.put("AWE","AT&T WIRELS SVCS");
        companyNamesAndSymbols.put("AXP","AMER EXPRESS CO");
        companyNamesAndSymbols.put("AYE","ALLEGHENY ENERGY");
        companyNamesAndSymbols.put("AZO","AUTOZONE INC");
        companyNamesAndSymbols.put("BA","BOEING CO");
        companyNamesAndSymbols.put("BAC","BANK OF AMERICA");
        companyNamesAndSymbols.put("BAX","BAXTER INTL INC");
        companyNamesAndSymbols.put("BBBY","BED BATH BEYOND");
        companyNamesAndSymbols.put("BBT","BB&T CORP");
        companyNamesAndSymbols.put("BBY","BEST BUY CO INC");
        companyNamesAndSymbols.put("BC","BRUNSWICK CORP");
        companyNamesAndSymbols.put("BCC","BOISE CASCADE");
        companyNamesAndSymbols.put("BCR","C R BARD");
        companyNamesAndSymbols.put("BDK","BLACK & DECKER");
        companyNamesAndSymbols.put("BDX","BECTON DICKINSN");
        companyNamesAndSymbols.put("BEN","FRANKLIN RES");
        companyNamesAndSymbols.put("BFb","BROWN FORMAN B");
        companyNamesAndSymbols.put("BHI","BAKER HUGHES INC");
        companyNamesAndSymbols.put("BIIB","BIOGEN IDEC");
        companyNamesAndSymbols.put("BJS","BJ SERVICES CO");
        companyNamesAndSymbols.put("BK","BANK OF NEW YORK");
        companyNamesAndSymbols.put("BLI","BIG LOTS INC");
        companyNamesAndSymbols.put("BLL","BALL CORP");
        companyNamesAndSymbols.put("BLS","BELLSOUTH CORP");
        companyNamesAndSymbols.put("BMC","BMC SOFTWARE");
        companyNamesAndSymbols.put("BMET","BIOMET INC");
        companyNamesAndSymbols.put("BMS","BEMIS COMPANY");
        companyNamesAndSymbols.put("BMY","BRISTOL MYERS SQ"); 
        companyNamesAndSymbols.put("BNI","BURL NTHN SANTA");
        companyNamesAndSymbols.put("BOL","BAUSCH & LOMB");
        companyNamesAndSymbols.put("BR","BURLINGTON RES");
        companyNamesAndSymbols.put("BRCM","BROADCOM CORP");
        companyNamesAndSymbols.put("BSC","BEAR STEARNS COS");
        companyNamesAndSymbols.put("BSX","BOSTON SCIEN CP");
        companyNamesAndSymbols.put("BUD","ANHEUSER BUSCH");
        companyNamesAndSymbols.put("C","CITIGROUP");
        companyNamesAndSymbols.put("CA","COMPUTER ASSOC");
        companyNamesAndSymbols.put("CAG","CONAGRA FOODS");
        companyNamesAndSymbols.put("CAH","CARDINAL HLTH");
        companyNamesAndSymbols.put("CAT","CATERPILLAR INC");
        companyNamesAndSymbols.put("CB","CHUBB CORP");
        companyNamesAndSymbols.put("CBE","COOPER INDS LTD");
        companyNamesAndSymbols.put("CC","CIRCUIT CITY");
        companyNamesAndSymbols.put("CCE","COCA COLA ENT");
        companyNamesAndSymbols.put("CCL","CARNIVAL CORP");
        companyNamesAndSymbols.put("CCU","CLEAR CHANNEL");
        companyNamesAndSymbols.put("CD","CENDANT CP");
        companyNamesAndSymbols.put("CEG","CONSTELL ENERGY");
        companyNamesAndSymbols.put("CFC","COUNTRYWIDE FNCL");
        companyNamesAndSymbols.put("CHIR","CHIRON CORP");
        companyNamesAndSymbols.put("CI","CIGNA CORP");
        companyNamesAndSymbols.put("CIEN","CIENA CORP");
        companyNamesAndSymbols.put("CIN","CINERGY CORP"); 
        companyNamesAndSymbols.put("CINF","CINCINNATI FIN");
        companyNamesAndSymbols.put("CL","COLGATE PALMOLIV");
        companyNamesAndSymbols.put("CLX","CLOROX CO");
        companyNamesAndSymbols.put("CMA","COMERICA INC");
        companyNamesAndSymbols.put("CMCSA","COMCAST CORP A");
        companyNamesAndSymbols.put("CMI","CUMMINS INC");
        companyNamesAndSymbols.put("CMS","CMS ENERGY CORP"); 
        companyNamesAndSymbols.put("CMVT","COMVERSE TECH");
        companyNamesAndSymbols.put("CMX","CAREMARK RX");
        companyNamesAndSymbols.put("CNP","CENTERPOINT");
        companyNamesAndSymbols.put("COF","CAP ONE FINAN");
        companyNamesAndSymbols.put("COH","COACH INC");
        companyNamesAndSymbols.put("COL","ROCKWELL COLL");
        companyNamesAndSymbols.put("COP","CONOCOPHILLIPS");
        companyNamesAndSymbols.put("COST","COSTCO WHOLESAL");
        companyNamesAndSymbols.put("CPB","CAMPBELL SOUP CO");
        companyNamesAndSymbols.put("CPN","CALPINE CORP");
        companyNamesAndSymbols.put("CPWR","COMPUWARE CORP");
        companyNamesAndSymbols.put("CR","CRANE CO");
        companyNamesAndSymbols.put("CSC","COMPUTER SCIENCE");
        companyNamesAndSymbols.put("CSCO","CISCO SYSTEMS");
        companyNamesAndSymbols.put("CSX","CSX CORP");
        companyNamesAndSymbols.put("CTAS","CINTAS CORP");
        companyNamesAndSymbols.put("CTB","COOPER TIRE & RB");
        companyNamesAndSymbols.put("CTL","CENTURYTEL INC");
        companyNamesAndSymbols.put("CTX","CENTEX CORP");
        companyNamesAndSymbols.put("CTXS","CITRIX SYSTEMS");
        companyNamesAndSymbols.put("CVG","CONVERGYS CP");
        companyNamesAndSymbols.put("CVS","CVS CORPORATION");
        companyNamesAndSymbols.put("CVX","CHEVRONTEXACO");
        companyNamesAndSymbols.put("CZN","CITIZENS COMMS");
        companyNamesAndSymbols.put("D","DOMINION RES INC");
        companyNamesAndSymbols.put("DAL","DELTA AIR LINES");
        companyNamesAndSymbols.put("DCN","DANA CORP");
        companyNamesAndSymbols.put("DD","DU PONT CO");
        companyNamesAndSymbols.put("DDS","DILLARD CL A");
        companyNamesAndSymbols.put("DE","DEERE & CO");
        companyNamesAndSymbols.put("DELL","DELL INC");
        companyNamesAndSymbols.put("DG","DOLLAR GEN");
        companyNamesAndSymbols.put("DGX","QUEST DIAG");
        companyNamesAndSymbols.put("DHR","DANAHER");
        companyNamesAndSymbols.put("DIS","WALT DISNEY CO");
        companyNamesAndSymbols.put("DJ","DOW JONES & CO");
        companyNamesAndSymbols.put("DLX","DELUXE CORP");
        companyNamesAndSymbols.put("DOV","DOVER CORP");
        companyNamesAndSymbols.put("DOW","DOW CHEMICAL");
        companyNamesAndSymbols.put("DPH","DELPHI CORP");
        companyNamesAndSymbols.put("DRI","DARDEN REST");
        companyNamesAndSymbols.put("DTE","DTE ENERGY");
        companyNamesAndSymbols.put("DUK","DUKE ENERGY");
        companyNamesAndSymbols.put("DVN","DEVON ENERGY");
        companyNamesAndSymbols.put("DYN","DYNEGY INC");
        companyNamesAndSymbols.put("EBAY","EBAY INC");
        companyNamesAndSymbols.put("EC","ENGELHARD CORP");
        companyNamesAndSymbols.put("ECL","ECOLAB INC"); 
        companyNamesAndSymbols.put("ED","CONSOL EDISON"); 
        companyNamesAndSymbols.put("EDS","ELECTR DATA"); 
        companyNamesAndSymbols.put("EFX","EQUIFAX INC");
        companyNamesAndSymbols.put("EIX","EDISON INTL");
        companyNamesAndSymbols.put("EK","EASTMAN KODAK"); 
        companyNamesAndSymbols.put("EMC","EMC CORP");
        companyNamesAndSymbols.put("EMN","EASTMAN CHEM"); 
        companyNamesAndSymbols.put("EMR","EMERSON ELECTRIC");
        companyNamesAndSymbols.put("EOG","EOG RESOURCES"); 
        companyNamesAndSymbols.put("EOP","EQUITY OFFICE"); 
        companyNamesAndSymbols.put("EP","EL PASO CORP"); 
        companyNamesAndSymbols.put("EQR","EQ RESIDENT"); 
        companyNamesAndSymbols.put("ERTS","ELECTRONIC ART"); 
        companyNamesAndSymbols.put("ESRX","EXPRESS SCRIPTS");
        companyNamesAndSymbols.put("ET","E*TRADE FINCL CP");
        companyNamesAndSymbols.put("ETN","EATON");
        companyNamesAndSymbols.put("ETR","ENTERGY CP");
        companyNamesAndSymbols.put("EXC","EXELON CORP");
        companyNamesAndSymbols.put("F","FORD MOTOR CO");
        companyNamesAndSymbols.put("FCX","FRPRT-MCM GD");
        companyNamesAndSymbols.put("FD","FED DEPT STRS"); 
        companyNamesAndSymbols.put("FDC","FIRST DATA CORP");
        companyNamesAndSymbols.put("FDO","FAMILY DLR STRS"); 
        companyNamesAndSymbols.put("FDX","FEDEX CORP");
        companyNamesAndSymbols.put("FE","FIRSTENERGY");
        companyNamesAndSymbols.put("FHN","FIRST HORIZN NTL");
        companyNamesAndSymbols.put("FII","FED INVESTORS"); 
        companyNamesAndSymbols.put("FISV","FISERV INC"); 
        companyNamesAndSymbols.put("FITB","FIFTH THR BNCP");
        companyNamesAndSymbols.put("FLR","FLUOR CORP");
        companyNamesAndSymbols.put("FNM","FANNIE MAE");
        companyNamesAndSymbols.put("FO","FORTUNE BRANDS");
        companyNamesAndSymbols.put("FON","SPRINT FON GP"); 
        companyNamesAndSymbols.put("FPL","FPL GROUP INC");
        companyNamesAndSymbols.put("FRE","FREDDIE MAC"); 
        companyNamesAndSymbols.put("FRX","FOREST LABS");
        companyNamesAndSymbols.put("FSH","FISHER SCI INTL");
        companyNamesAndSymbols.put("G","GILLETTE CO"); 
        companyNamesAndSymbols.put("GAS","NICOR INC");
        companyNamesAndSymbols.put("GCI","GANNETT CO INC"); 
        companyNamesAndSymbols.put("GD","GENERAL DYNAMICS");
        companyNamesAndSymbols.put("GDT","GUIDANT CORP");
        companyNamesAndSymbols.put("GDW","GOLDEN WEST FIN"); 
        companyNamesAndSymbols.put("GE","GENERAL ELEC CO");
        companyNamesAndSymbols.put("GENZ","GENZYME GEN");
        companyNamesAndSymbols.put("GILD","GILEAD SCI");
        companyNamesAndSymbols.put("GIS","GENERAL MILLS");
        companyNamesAndSymbols.put("GLK","GREAT LAKES CHEM");
        companyNamesAndSymbols.put("GLW","CORNING INC");
        companyNamesAndSymbols.put("GM","GENERAL MOTORS");
        companyNamesAndSymbols.put("GP","GEORGIA-PACIFIC");
        companyNamesAndSymbols.put("GPC","GENUINE PARTS CO");
        companyNamesAndSymbols.put("GPS","GAP INC");
        companyNamesAndSymbols.put("GR","GOODRICH CORP");
        companyNamesAndSymbols.put("GS","GOLDM SACHS GRP");
        companyNamesAndSymbols.put("GT","GOODYEAR TIRE"); 
        companyNamesAndSymbols.put("GTW","GATEWAY INC");
        companyNamesAndSymbols.put("GWW","W W GRAINGER INC"); 
        companyNamesAndSymbols.put("HAL","HALLIBURTON CO");
        companyNamesAndSymbols.put("HAS","HASBRO INC"); 
        companyNamesAndSymbols.put("HBAN","HUNTGTN BKSHR"); 
        companyNamesAndSymbols.put("HCA","HCA INC");
        companyNamesAndSymbols.put("HCR","MANOR CARE INC"); 
        companyNamesAndSymbols.put("HD","HOME DEPOT INC");
        companyNamesAndSymbols.put("HDI","HARLEY-DAVIDSON");
        companyNamesAndSymbols.put("HET","HARRAHS ENTER");
        companyNamesAndSymbols.put("HIG","HARTFORD FINL");
        companyNamesAndSymbols.put("HLT","HILTON HOTELS");
        companyNamesAndSymbols.put("HMA","HEALTH MGMT");
        companyNamesAndSymbols.put("HNZ","H J HEINZ CO"); 
        companyNamesAndSymbols.put("HON","HONEYWELL INTL");
        companyNamesAndSymbols.put("HOT","STARWOOD HOTELS"); 
        companyNamesAndSymbols.put("HPC","HERCULES INC");
        companyNamesAndSymbols.put("HPQ","HEWLETT-PACKARD");
        companyNamesAndSymbols.put("HRB","H & R BLOCK INC"); 
        companyNamesAndSymbols.put("HSP","HOSPIRA");
        companyNamesAndSymbols.put("HSY","HERSHEY FOODS CP");
        companyNamesAndSymbols.put("HUM","HUMANA INC"); 
        companyNamesAndSymbols.put("IBM","INTL BUS MACHINE"); 
        companyNamesAndSymbols.put("IFF","INTL FLAV & FRAG"); 
        companyNamesAndSymbols.put("IGT","INTL GAME TECH"); 
        companyNamesAndSymbols.put("INTC","INTEL CORP"); 
        companyNamesAndSymbols.put("INTU","INTUIT INC"); 
        companyNamesAndSymbols.put("IP","INTL PAPER CO");
        companyNamesAndSymbols.put("IPG","INTERPUBLIC GRP");
        companyNamesAndSymbols.put("IR","INGERSOLL-RAND");
        companyNamesAndSymbols.put("ITT","ITT INDS INC");
        companyNamesAndSymbols.put("ITW","ILLINOIS TOOL WK"); 
        companyNamesAndSymbols.put("JBL","JABIL CIRCUIT");
        companyNamesAndSymbols.put("JCI","JOHNSON CONTROLS");
        companyNamesAndSymbols.put("JCP","J C PENNEY CO"); 
        companyNamesAndSymbols.put("JDSU","JDS UNIPHASE");
        companyNamesAndSymbols.put("JNJ","JOHNSON&JOHNSON");
        companyNamesAndSymbols.put("JNS","JANUS CAPITAL GP");
        companyNamesAndSymbols.put("JNY","JONES APPAREL");
        companyNamesAndSymbols.put("JP","JEFFERSON PILOT");
        companyNamesAndSymbols.put("JPM","JPMORGAN CHASE");
        companyNamesAndSymbols.put("JWN","NORDSTROM INC");
        companyNamesAndSymbols.put("K","KELLOGG CO");
        companyNamesAndSymbols.put("KBH","KB HOME");
        companyNamesAndSymbols.put("KEY","KEYCORP NEW");
        companyNamesAndSymbols.put("KG","KING PHARM");
        companyNamesAndSymbols.put("KLAC","KLA TENCOR");
        companyNamesAndSymbols.put("KMB","KIMBERLY-CLARK");
        companyNamesAndSymbols.put("KMG","KERR MCGEE CORP");
        companyNamesAndSymbols.put("KMI","KINDER MORGAN");
        companyNamesAndSymbols.put("KO","COCA-COLA CO");
        companyNamesAndSymbols.put("KR","KROGER CO");
        companyNamesAndSymbols.put("KRB","MBNA CORP");
        companyNamesAndSymbols.put("KRI","KNIGHT RIDDER");
        companyNamesAndSymbols.put("KSE","KEYSPAN CORP");
        companyNamesAndSymbols.put("KSS","KOHLS CORP"); 
        companyNamesAndSymbols.put("LEG","LEGGET & PLATT");
        companyNamesAndSymbols.put("LEH","LEHMAN BROS");
        companyNamesAndSymbols.put("LIZ","LIZ CLAIBORNE");
        companyNamesAndSymbols.put("LLTC","LINEAR TECH");
        companyNamesAndSymbols.put("LLY","ELI LILLY");
        companyNamesAndSymbols.put("LMT","LOCKHEED MARTIN");
        companyNamesAndSymbols.put("LNC","LINCOLN NATL");
        companyNamesAndSymbols.put("LOW","LOWES COMPANIES");
        companyNamesAndSymbols.put("LPX","LOUISIANA PACIF");
        companyNamesAndSymbols.put("LSI","LSI LOGIC");
        companyNamesAndSymbols.put("LTD","LIMITED BRANDS");
        companyNamesAndSymbols.put("LTR","LOEWS CORP"); 
        companyNamesAndSymbols.put("LU","LUCENT TECH");
        companyNamesAndSymbols.put("LUV","SW AIRLINES"); 
        companyNamesAndSymbols.put("LXK","LEXMARK INTL");
        companyNamesAndSymbols.put("MAR","MARRIOTT INTL");
        companyNamesAndSymbols.put("MAS","MASCO CORP");
        companyNamesAndSymbols.put("MAT","MATTEL INC");
        companyNamesAndSymbols.put("MAY","MAY DEPT STORES");
        companyNamesAndSymbols.put("MBI","MBIA INC");
        companyNamesAndSymbols.put("MCD","MCDONALDS CORP");
        companyNamesAndSymbols.put("MCK","MCKESSON CORP");
        companyNamesAndSymbols.put("MCO","MOODY'S CORP");
        companyNamesAndSymbols.put("MDP","MEREDITH CORP");
        companyNamesAndSymbols.put("MDT","MEDTRONIC INC");
        companyNamesAndSymbols.put("MEDI","MEDIMMUNE INC");
        companyNamesAndSymbols.put("MEL","MELLON FINL CORP");
        companyNamesAndSymbols.put("MER","MERRILL LYNCH");
        companyNamesAndSymbols.put("MERQ","MERCURY INTRACT");
        companyNamesAndSymbols.put("MET","METLIFE INC"); 
        companyNamesAndSymbols.put("MHP","MCGRAW HILL COS");
        companyNamesAndSymbols.put("MHS","MEDCO HLTH SOLN");
        companyNamesAndSymbols.put("MI","MARSHALL ILSLEY");
        companyNamesAndSymbols.put("MIL","MILLIPORE CP");
        companyNamesAndSymbols.put("MKC","MCCORMICK & CO");
        companyNamesAndSymbols.put("MMC","MARSH & MCLENNAN");
        companyNamesAndSymbols.put("MMM","3M COMPANY");
        companyNamesAndSymbols.put("MNST","MONSTER WRLDWIDE");
        companyNamesAndSymbols.put("MO","ALTRIA GROUP");
        companyNamesAndSymbols.put("MOLX","MOLEX INC");
        companyNamesAndSymbols.put("MON","MONSANTO CO");
        companyNamesAndSymbols.put("MOT","MOTOROLA INC"); 
        companyNamesAndSymbols.put("MRK","MERCK & CO");
        companyNamesAndSymbols.put("MRO","MARATHON OIL");
        companyNamesAndSymbols.put("MSFT","MICROSOFT CP");
        companyNamesAndSymbols.put("MTB","M&T BANK CORP");
        companyNamesAndSymbols.put("MTG","MGIC INV CP");
        companyNamesAndSymbols.put("MU","MICRON TECH");
        companyNamesAndSymbols.put("MWD","MORGAN STANLEY");
        companyNamesAndSymbols.put("MWV","MEADWESTVACO CP");
        companyNamesAndSymbols.put("MXIM","MAXIM INTEGRTD");
        companyNamesAndSymbols.put("MYG","MAYTAG CORP");
        companyNamesAndSymbols.put("MYL","MYLAN LABS");
        companyNamesAndSymbols.put("NAV","NAVISTAR INTL");
        companyNamesAndSymbols.put("NBR","NABORS INDS LTD");
        companyNamesAndSymbols.put("NCC","NATIONAL CITY"); 
        companyNamesAndSymbols.put("NCR","NCR CORP");
        companyNamesAndSymbols.put("NE","NOBLE CORP");
        companyNamesAndSymbols.put("NEM","NEWMONT MINING");
        companyNamesAndSymbols.put("NFB","N FORK BANCP");
        companyNamesAndSymbols.put("NI","NISOURCE INC");
        companyNamesAndSymbols.put("NKE","NIKE INC CL B");
        companyNamesAndSymbols.put("NOC","NORTHROP GRUMMAN");
        companyNamesAndSymbols.put("NOVL","NOVELL INC"); 
        companyNamesAndSymbols.put("NSC","NORFOLK SOUTHERN");
        companyNamesAndSymbols.put("NSM","NATL SEMICONDUCT");
        companyNamesAndSymbols.put("NTAP","NETWK APPLIANCE");
        companyNamesAndSymbols.put("NTRS","NORTHERN TRUST");
        companyNamesAndSymbols.put("NUE","NUCOR");
        companyNamesAndSymbols.put("NVDA","NVIDIA CORP");
        companyNamesAndSymbols.put("NVLS","NOVELLUS SYS");
        companyNamesAndSymbols.put("NWL","NEWELL RUBBERMD"); 
        companyNamesAndSymbols.put("NXTL","NEXTEL COMMS");
        companyNamesAndSymbols.put("NYT","NY TIMES");
        companyNamesAndSymbols.put("ODP","OFFICE DEPOT");
        companyNamesAndSymbols.put("OMC","OMNICOM GP INC");
        companyNamesAndSymbols.put("ORCL","ORACLE CORP");
        companyNamesAndSymbols.put("OXY","OCCIDENTAL PETE");
        companyNamesAndSymbols.put("PAYX","PAYCHEX INC");
        companyNamesAndSymbols.put("PBG","PEPSI BOTTLING");
        companyNamesAndSymbols.put("PBI","PITNEY BOWES INC");
        companyNamesAndSymbols.put("PCAR","PACCAR INC");
        companyNamesAndSymbols.put("PCG","PG&E CORP");
        companyNamesAndSymbols.put("PCL","PLUM CREEK TIMB");
        companyNamesAndSymbols.put("PD","PHELPS DODGE CP");
        companyNamesAndSymbols.put("PEG","PUBL SVC ENTER"); 
        companyNamesAndSymbols.put("PEP","PEPSICO INC");
        companyNamesAndSymbols.put("PFE","PFIZER INC");
        companyNamesAndSymbols.put("PFG","PRINCIPAL FINL");
        companyNamesAndSymbols.put("PG","PROCTER & GAMBLE");
        companyNamesAndSymbols.put("PGL","PEOPLES ENERGY");
        companyNamesAndSymbols.put("PGN","PROGRESS ENERGY");
        companyNamesAndSymbols.put("PGR","THE PROGRESSIVE");
        companyNamesAndSymbols.put("PH","PARKER-HANNIFIN");
        companyNamesAndSymbols.put("PHM","PULTE HOMES INC");
        companyNamesAndSymbols.put("PKI","PERKINELMER");
        companyNamesAndSymbols.put("PLD","PROLOGIS");
        companyNamesAndSymbols.put("PLL","PALL CORP");
        companyNamesAndSymbols.put("PMCS","PMC-SIERRA INC");
        companyNamesAndSymbols.put("PMTC","PARAMETRIC");
        companyNamesAndSymbols.put("PNC","PNC FINL SVC");
        companyNamesAndSymbols.put("PNW","PINNACL WEST CAP");
        companyNamesAndSymbols.put("PPG","PPG IND");
        companyNamesAndSymbols.put("PPL","PPL CORP");
        companyNamesAndSymbols.put("PRU","PRUDENTIAL FINL");
        companyNamesAndSymbols.put("PSFT","PEOPLESOFT INC");
        companyNamesAndSymbols.put("PTV","PACTIV CORP");
        companyNamesAndSymbols.put("PVN","PROV FIN");
        companyNamesAndSymbols.put("PWER","POWER ONE");
        companyNamesAndSymbols.put("PX","PRAXAIR INC");
        companyNamesAndSymbols.put("Q","QWEST COMMS INTL"); 
        companyNamesAndSymbols.put("QCOM","QUALCOMM INC");
        companyNamesAndSymbols.put("QLGC","QLOGIC CORP");
        companyNamesAndSymbols.put("R","RYDER SYSTEM INC");
        companyNamesAndSymbols.put("RAI","REYNOLDS AMRICAN");
        companyNamesAndSymbols.put("RBK","REEBOK INTL LTD");
        companyNamesAndSymbols.put("RDC","ROWAN CO INC");
        companyNamesAndSymbols.put("RF","REGIONS FINANCL");
        companyNamesAndSymbols.put("RHI","ROB HALF INTL");
        companyNamesAndSymbols.put("RIG","TRANSOCEAN INC");
        companyNamesAndSymbols.put("RKY","ADOLPH COORS");
        companyNamesAndSymbols.put("ROH","ROHM & HAAS CO");
        companyNamesAndSymbols.put("ROK","ROCKWELL AUTOMAT"); 
        companyNamesAndSymbols.put("RRD","DONNELLEY & SONS");
        companyNamesAndSymbols.put("RSH","RADIOSHACK");
        companyNamesAndSymbols.put("RTN","RAYTHEON CO");
        companyNamesAndSymbols.put("RX","IMS HEALTH");
        companyNamesAndSymbols.put("S","SEARS ROEBUCK");
        companyNamesAndSymbols.put("SAFC","SAFECO CORP");
        companyNamesAndSymbols.put("SANM","SANMINA-SCI CP"); 
        companyNamesAndSymbols.put("SBC","SBC COMMS");
        companyNamesAndSymbols.put("SBL","SYMBOL TECH");
        companyNamesAndSymbols.put("SBUX","STARBUCKS CORP");
        companyNamesAndSymbols.put("SCH","CHARLES SCHWAB");
        companyNamesAndSymbols.put("SDS","SUNGARD DATA SYS");
        companyNamesAndSymbols.put("SEBL","SIEBEL SYSTEMS");
        companyNamesAndSymbols.put("SEE","SEALED AIR CP");
        companyNamesAndSymbols.put("SFA","SCIENTIFIC ATL");
        companyNamesAndSymbols.put("SGP","SCHERING-PLOUGH");
        companyNamesAndSymbols.put("SHW","SHERWIN-WILLIAMS");
        companyNamesAndSymbols.put("SIAL","SIGMA ALDRICH");
        companyNamesAndSymbols.put("SLB","SCHLUMBERGER LTD");
        companyNamesAndSymbols.put("SLE","SARA LEE CORP"); 
        companyNamesAndSymbols.put("SLM","SLM CORPORATION");
        companyNamesAndSymbols.put("SLR","SOLECTRON");
        companyNamesAndSymbols.put("SNA","SNAP-ON INC");
        companyNamesAndSymbols.put("SNV","SYNOVUS FINAN");
        companyNamesAndSymbols.put("SO","SOUTHERN CO");
        companyNamesAndSymbols.put("SOTR","SOUTHTRUST CP");
        companyNamesAndSymbols.put("SOV","SOVRGN BANCRP");
        companyNamesAndSymbols.put("SPG","SIMON PROP GRP");
        companyNamesAndSymbols.put("SPLS","STAPLES INC");
        companyNamesAndSymbols.put("SRE","SEMPRA ENERGY");
        companyNamesAndSymbols.put("STA","ST PAUL TRAVLRS");
        companyNamesAndSymbols.put("STI","SUNTRUST BKS");
        companyNamesAndSymbols.put("STJ","ST JUDE MEDICAL");
        companyNamesAndSymbols.put("STT","ST STREET CP"); 
        companyNamesAndSymbols.put("SUN","SUNOCO INC");
        companyNamesAndSymbols.put("SUNW","SUN MICROSYS"); 
        companyNamesAndSymbols.put("SVU","SUPERVALU INC");
        companyNamesAndSymbols.put("SWK","STANLEY WORKS");
        companyNamesAndSymbols.put("SWY","SAFEWAY INC");
        companyNamesAndSymbols.put("SYK","STRYKER");
        companyNamesAndSymbols.put("SYMC","SYMANTEC CORP");
        companyNamesAndSymbols.put("SYY","SYSCO CORP");
        companyNamesAndSymbols.put("T","AT&T CORP");
        companyNamesAndSymbols.put("TE","TECO ENERGY");
        companyNamesAndSymbols.put("TEK","TEKTRONIX");
        companyNamesAndSymbols.put("TER","TERADYNE INC");
        companyNamesAndSymbols.put("TGT","TARGET CORP");
        companyNamesAndSymbols.put("THC","TENET HEALTHCR");
        companyNamesAndSymbols.put("TIF","TIFFANY & CO");
        companyNamesAndSymbols.put("TIN","TEMPLE INLAND");
        companyNamesAndSymbols.put("TJX","TJX CO INC");
        companyNamesAndSymbols.put("TLAB","TELLABS INC");
        companyNamesAndSymbols.put("TMK","TORCHMARK CORP");
        companyNamesAndSymbols.put("TMO","THERMO ELECTRON");
        companyNamesAndSymbols.put("TOY","TOYS R US CORP");
        companyNamesAndSymbols.put("TRB","TRIBUNE CO");
        companyNamesAndSymbols.put("TROW","T ROWE PRICE");
        companyNamesAndSymbols.put("TSG","SABRE HLDGS");
        companyNamesAndSymbols.put("TWX","TIME WARNER INC");
        companyNamesAndSymbols.put("TXN","TEXAS INSTRUMENT");
        companyNamesAndSymbols.put("TXT","TEXTRON INC");
        companyNamesAndSymbols.put("TXU","TXU CORP");
        companyNamesAndSymbols.put("TYC","TYCO INTL");
        companyNamesAndSymbols.put("UCL","UNOCAL CORP DEL");
        companyNamesAndSymbols.put("UIS","UNISYS CORP");
        companyNamesAndSymbols.put("UNH","UNITEDHEALTH GP");
        companyNamesAndSymbols.put("UNM","UNUMPROVIDENT");
        companyNamesAndSymbols.put("UNP","UNION PACIFIC CP"); 
        companyNamesAndSymbols.put("UPS","UNITED PARCEL B");
        companyNamesAndSymbols.put("USB","US BANCORP");
        companyNamesAndSymbols.put("UST","UST INC");
        companyNamesAndSymbols.put("UTX","UNITED TECH CP");
        companyNamesAndSymbols.put("UVN","UNIVISION COMM");
        companyNamesAndSymbols.put("VC","VISTEON CORP");
        companyNamesAndSymbols.put("VFC","VF CORP");
        companyNamesAndSymbols.put("VIAb","VIACOM CL B");
        companyNamesAndSymbols.put("VLO","VALERO ENERGY");
        companyNamesAndSymbols.put("VMC","VULCAN MATRLS");
        companyNamesAndSymbols.put("VRTS","VERITAS SOFTWARE");
        companyNamesAndSymbols.put("VZ","VERIZON COMMS");
        companyNamesAndSymbols.put("WAG","WALGREEN CO");
        companyNamesAndSymbols.put("WAT","WATERS CORP");
        companyNamesAndSymbols.put("WB","WACHOVIA CORP"); 
        companyNamesAndSymbols.put("WEN","WENDYS INTL");
        companyNamesAndSymbols.put("WFC","WELLS FARGO & CO");
        companyNamesAndSymbols.put("WHR","WHIRL POOL CORP");
        companyNamesAndSymbols.put("WIN","WINN-DIXIE STRS");
        companyNamesAndSymbols.put("WLP","WELLPNT HLTH NET"); 
        companyNamesAndSymbols.put("WM","WASHINGTN MUTUAL");
        companyNamesAndSymbols.put("WMB","WILLIAMS COMPS");
        companyNamesAndSymbols.put("WMI","WASTE MANAGEMNT");
        companyNamesAndSymbols.put("WMT","WAL-MART STORES");
        companyNamesAndSymbols.put("WOR","WORTHINGTON INDS");
        companyNamesAndSymbols.put("WPI","WATSON PHARM");
        companyNamesAndSymbols.put("WWY","WM WRIGLEY JR");
        companyNamesAndSymbols.put("WY","WEYERHAEUSER CO");
        companyNamesAndSymbols.put("WYE","WYETH");
        companyNamesAndSymbols.put("X","US STEEL CORP");
        companyNamesAndSymbols.put("XEL","XCEL ENERGY");
        companyNamesAndSymbols.put("XL","XL CAPITAL LTD"); 
        companyNamesAndSymbols.put("XLNX","XILINX INC");
        companyNamesAndSymbols.put("XOM","EXXON MOBIL");
        companyNamesAndSymbols.put("XRX","XEROX CORP");
        companyNamesAndSymbols.put("YHOO","YAHOO! INC");
        companyNamesAndSymbols.put("YUM","YUM! BRANDS INC");
        companyNamesAndSymbols.put("ZION","ZIONS BANCORP");
        companyNamesAndSymbols.put("ZMH","ZIMMER HLDGS");

        return companyNamesAndSymbols;
    }

    public static void main(String args[])
    {
        try
        {
            if(args.length == 0)
            {
                usage();
            }
            else
            {
                if(args[0]==null)
                {
                    output("No TopicConnectionFactory specified. Exiting");
                    usage();
                }

                if((args[1].equalsIgnoreCase("FTSE"))|(args[1].equalsIgnoreCase("FTSE"))|(args[1].equalsIgnoreCase("FTSE")))
                {
                    stockExchange=args[1];
                    output("StockExchange is "+stockExchange);
                    output("Set jndiICF to "+icf);      
                }
                else
                {
                    icf=args[1];
                    output("Set jndiICF to "+icf);      
                }

                if(args.length>2)
                {
                    url=args[2];
                    output("Set jndiURL to "+url);  
                }

                if(args.length>3)
                {
                    stockExchange=args[3];
                    output("StockExchange is "+stockExchange);  
                }

                StockTickerPublisher ticker = new StockTickerPublisher(args);

            }
        }
        catch(Exception e)
        {
            usage();
        }
    }
}
