//=============================================================================
//
// IBM WBI Brokers Replay
//
// Licensed Materials - Property of IBM Corp.
// 5648-C63 (c) Copyright IBM Corp. 2004
//
//=============================================================================

package samples;

import java.sql.*;
import java.io.*;
import java.text.*;
import java.util.*;

import javax.jms.*;
import javax.naming.*;
import javax.naming.directory.*;

public class TennisPublisherSimulator
{
    java.sql.Connection con = null;
    String publishString = null;
    String gamesPlayedPrefix;
    String setsPlayedPrefix;

    static StdInThread sts = null;

    String[] currentGameScore = { "0", "0"};
    int[] currentSetScore = { 0, 0};
    int[] setsScore = { 0, 0};
    int[] tieBreak = { 0, 0};
    int numberOfGamesPlayedInThisSet = 0;
    int numberOfSetsPlayed = 0;
    int p1_numberOfFirstServesIn = 0;
    int p1_numberOfSecondServesIn = 0;
    int p2_numberOfFirstServesIn = 0;
    int p2_numberOfSecondServesIn = 0;
    int p1_breakPointsSaved = 0;
    int totalNumberOfGamesPlayed = 0;

    public final String TRANSPORT_TCF = "Replayer_TCF";
    static String icf = "com.sun.jndi.fscontext.RefFSContextFactory";
    static String url = "file:/C:/jndi";

    // Required JMS object declarations
    Context ctx = null;
    TopicConnectionFactory tcf = null;
    Topic topic = null;
    static TopicConnection conn = null;
    TopicSession session = null;
    TopicPublisher pub = null;
    static int delay;

    public void insertGameSimulation(String args[])
    {
        String randomCourt;

        String[] malePlayers =
        {
            "Roger Federer",
            "Andy Roddick",
            "Guillermo Coria",
            "Juan Carlos Ferrero",
            "Carlos Moya",
            "Andre Agassi",
            "Rainer Schuerrler",
            "Tim Henman",
            "Sebastien Grosjean",
            "Nicolas Massu",
            "Leyton Hewitt",
            "Paradorn Scrichaphan",
            "Jiri Novak",
            "Sjeng Schalken",
            "Fernando Gonzales"};

        String[] femalePlayers =
        {
            "Justine Henin-Hardenne",
            "Serena Williams",
            "Amelie Mauresmo",
            "Venus Williams",
            "Lindsay Davenport",
            "Anastasia Myskina",
            "Jennifer Capriati",
            "Nadia Petrova",
            "Elena Dementieva",
            "Vera Zvonareva",
            "Svetlana Kuznetsova",
            "Ai Sugiyama",
            "Chanda Rubin",
            "Paola Suarez",
            "Silvia Farina Elia",
            "Patty Schnyder"};

        String[] Court = { "Centre Court", "Court Number 1", "Court Number 2", "Court Number 3", "Court Number 4"};

        int firstOrSecondServe = 0;
        int firstServeSpeed = 0;
        int secondServeSpeed = 0;
        int shotwinner=0;

        String randomMalePlayer1 = null;
        String randomMalePlayer2 = null;

        String gameTopic = null;

        int gameScorePlayer1 = 0;
        int gameScorePlayer2 = 0;

        try
        {
            // Create a Hashtable for JNDI required parameters
            Hashtable env = new Hashtable();
            env.put(Context.INITIAL_CONTEXT_FACTORY, icf);
            env.put(Context.PROVIDER_URL, url);
            env.put(Context.REFERRAL, "throw");

            // Initialise the JNDI Context
            ctx = new InitialDirContext(env);
        }
        catch(Exception e)
        {
            output("Unable to create JNDI context");
            e.printStackTrace();
            System.exit(0);
        }

        // Obtain the TCF from JNDI context
        try
        {
            tcf = (TopicConnectionFactory)ctx.lookup(args[0]);
        }
        catch(Exception e)
        {
            usage();
            e.printStackTrace();
            try
            {
                ctx.close();
            }
            catch(Exception ce)
            {
            }
            System.exit(0);
        }

        // Close our JNDI context
        try
        {
            ctx.close();
        }
        catch(Exception ce)
        {
        }

        // Create our JMS objects
        try
        {
            conn = tcf.createTopicConnection();
            output("Connection created");

            // Create TopicSession with required transactionality
            session = conn.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            output("Non-transacted Session created");
        }
        catch(JMSException e)
        {
            output("Unable to create JMS objects");
            e.printStackTrace();
            Exception le = e.getLinkedException();
            if(le != null)
                le.printStackTrace();

                // Close our JMS objects
            if(session != null)
                try
                {
                    session.close();
                }
                catch(JMSException je)
                {
                }
            if(conn != null)
                try
                {
                    conn.close();
                }
                catch(JMSException je)
                {
                }

            System.exit(0);
        }

        sts = new StdInThread();
        sts.start();

        // Generate a random Game, use a do loop just in case a player ends up playing with himself(!)

        do
        {
            randomMalePlayer1 = malePlayers[((int) ((malePlayers.length) * Math.random()))];
            randomMalePlayer2 = malePlayers[((int) ((malePlayers.length) * Math.random()))];
        }
        while(randomMalePlayer1.equalsIgnoreCase(randomMalePlayer2));
        randomCourt = Court[((int) ((Court.length) * Math.random()))];

        gameTopic = randomMalePlayer1 + " vs " + randomMalePlayer2;

        publishString = "Game is  " + gameTopic;
        System.out.println(publishString);
        publishMessage(randomCourt, publishString);
        publishMessage(randomCourt, "Playing On " + randomCourt);

        java.util.Date currentDate = new java.util.Date();
        publishString = "Game started at  " + currentDate;
        publishMessage(randomCourt, publishString);

        // See who wins a point
        while(sts.isGoing())
        {
            whosServing(randomMalePlayer1, randomMalePlayer2, randomCourt);

            int pointWinner = (int) (2 * (Math.random()));

            switch(pointWinner)
            {
            case 0 :
                // Random Player 1 wins the point

                shotwinner = (int) (10 * (Math.random()));
                switch(shotwinner)
                {
                
                case 0 : publishString = randomMalePlayer1 + " wins the point with a forehand passing shot"; break;
                case 1 : publishString = randomMalePlayer1 + " wins the point with a backhand passing shot"; break;
                case 2 : publishString = randomMalePlayer1 + " wins the point with a topspin lob";break;
                case 3 : publishString = randomMalePlayer1 + " wins the point with a overhead smash";break;
                case 4 : publishString = randomMalePlayer1 + " wins the point with a forehand volley";break;
                case 5 : publishString = randomMalePlayer1 + " wins the point with a backhand volley";break;
                case 6 : publishString = randomMalePlayer1 + " wins the point as "+randomMalePlayer2+ " hits it into the net";break;
                case 7 : publishString = randomMalePlayer1 + " wins the point as "+randomMalePlayer2+ " hits it long";break;
                case 8 : publishString = randomMalePlayer1 + " wins the point as "+randomMalePlayer2+ " hits it wide";break;
                case 9 : publishString = randomMalePlayer1 + " wins the point with a dropshot";break;

                }

                publishMessage(randomCourt, publishString);

                try
                {
                    if(Integer.parseInt(currentGameScore[0]) == 0)
                    {
                        currentGameScore[0] = "15";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else if(Integer.parseInt(currentGameScore[0]) == 15)
                    {
                        currentGameScore[0] = "30";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);
                    }
                    else if(Integer.parseInt(currentGameScore[0]) == 30)
                    {
                        currentGameScore[0] = "40";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        // Player 1 is on 40 now. Check to see if this is a break point
                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else if((Integer.parseInt(currentGameScore[0]) == 40) & ((Integer.parseInt(currentGameScore[1]) == 40)))
                    {
                        currentGameScore[0] = "AD";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else
                    {
                        publishString = randomMalePlayer1 + " wins the game.";
                        publishMessage(randomCourt, publishString);

                        // Game has been won by player1
                        // Reset the game score
                        currentGameScore[0] = "0";
                        currentGameScore[1] = "0";

                        // Update the set score
                        currentSetScore[0]++;

                        if(!player1Serving(totalNumberOfGamesPlayed))
                        {

                            publishString = randomMalePlayer1 + " has broken " + randomMalePlayer2 + "s serve";
                            publishMessage(randomCourt, publishString);

                        }

                        // Update the running counters
                        numberOfGamesPlayedInThisSet = currentSetScore[0] + currentSetScore[1];
                        totalNumberOfGamesPlayed++;

                        // Work out which game this was
                        getGameNumberAsText(randomMalePlayer1, randomCourt);

                        publishString = "Total Number of games played = " + totalNumberOfGamesPlayed;
                        publishMessage(randomCourt, publishString);

                        publishString = "Score " + randomMalePlayer1 + ":" + currentSetScore[0] + "-" + currentSetScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        checkSetWon(randomMalePlayer1, randomMalePlayer2, randomCourt);
                        checkMatchWon(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                }
                catch(java.lang.NumberFormatException e)
                {
                    if(currentGameScore[0].equals("AD"))
                    {
                        currentGameScore[0] = "0";
                        currentGameScore[1] = "0";

                        publishString = randomMalePlayer1 + " wins the game.";
                        publishMessage(randomCourt, publishString);

                        if(!player1Serving(totalNumberOfGamesPlayed))
                        {

                            publishString = randomMalePlayer1 + " has broken " + randomMalePlayer2 + "s serve";
                            publishMessage(randomCourt, publishString);

                        }

                        // Update the set score
                        currentSetScore[0]++;
                        totalNumberOfGamesPlayed++;

                        numberOfGamesPlayedInThisSet = currentSetScore[0] + currentSetScore[1];
                        numberOfSetsPlayed = setsScore[0] + setsScore[1];

                        getGameNumberAsText(randomMalePlayer1, randomCourt);

                        publishString = "Score in games = " + currentSetScore[0] + "-" + currentSetScore[1];
                        publishMessage(randomCourt, publishString);

                        checkSetWon(randomMalePlayer1, randomMalePlayer2, randomCourt);
                        checkMatchWon(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }

                    else if(currentGameScore[1].equals("AD"))
                    {
                        currentGameScore[0] = "40";
                        currentGameScore[1] = "40";

                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);
                    }
                    else
                    {
                        currentGameScore[0] = "0";
                        currentGameScore[1] = "0";
                        System.out.println("ERROR");
                    }
                }

                break;

            case 1 :

                shotwinner = (int) (9 * (Math.random()));
                switch(shotwinner)
                {
                
                case 0 : publishString = randomMalePlayer2 + " wins the point with a forehand passing shot"; break;
                case 1 : publishString = randomMalePlayer2 + " wins the point with a backhand passing shot"; break;
                case 2 : publishString = randomMalePlayer2 + " wins the point with a topspin lob";break;
                case 3 : publishString = randomMalePlayer2 + " wins the point with a overhead smash";break;
                case 4 : publishString = randomMalePlayer2 + " wins the point with a forehand volley";break;
                case 5 : publishString = randomMalePlayer2 + " wins the point with a backhand volley";break;
                case 6 : publishString = randomMalePlayer2 + " wins the point as "+randomMalePlayer1+ " hits it into the net";break;
                case 7 : publishString = randomMalePlayer2 + " wins the point as "+randomMalePlayer1+ " hits it long";break;
                case 8 : publishString = randomMalePlayer2 + " wins the point as "+randomMalePlayer1+ " hits it wide";break;
                case 9 : publishString = randomMalePlayer2 + " wins the point with a dropshot";break;

                }

                publishMessage(randomCourt, publishString);

                try
                {

                    if(Integer.parseInt(currentGameScore[1]) == 0)
                    {
                        currentGameScore[1] = "15";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else if(Integer.parseInt(currentGameScore[1]) == 15)
                    {
                        currentGameScore[1] = "30";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else if(Integer.parseInt(currentGameScore[1]) == 30)
                    {
                        currentGameScore[1] = "40";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        // Player 2 is on 40 now. Check to see if this is a break point
                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else if((Integer.parseInt(currentGameScore[1]) == 40) & ((Integer.parseInt(currentGameScore[0]) == 40)))
                    {
                        currentGameScore[1] = "AD";
                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        isBreakPoint(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                    else
                    {
                        publishString = randomMalePlayer2 + " wins the game.";
                        publishMessage(randomCourt, publishString);

                        // Game has been won by player1
                        // Reset the game score
                        currentGameScore[0] = "0";
                        currentGameScore[1] = "0";

                        if(player1Serving(totalNumberOfGamesPlayed))
                        {

                            publishString = randomMalePlayer2 + " has broken " + randomMalePlayer1 + "s serve";
                            publishMessage(randomCourt, publishString);

                        }

                        // Update the set score
                        currentSetScore[1]++;

                        // Update the running counters
                        numberOfGamesPlayedInThisSet = currentSetScore[0] + currentSetScore[1];
                        totalNumberOfGamesPlayed++;

                        // Work out which game this was
                        getGameNumberAsText(randomMalePlayer2, randomCourt);

                        publishString = "Total Number of games played = " + totalNumberOfGamesPlayed;
                        publishMessage(randomCourt, publishString);

                        publishString = "Score " + randomMalePlayer1 + ":" + currentSetScore[0] + "-" + currentSetScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);

                        checkSetWon(randomMalePlayer1, randomMalePlayer2, randomCourt);
                        checkMatchWon(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }
                }
                catch(java.lang.NumberFormatException e)
                {
                    if(currentGameScore[1].equals("AD"))
                    {
                        currentGameScore[0] = "0";
                        currentGameScore[1] = "0";

                        publishString = randomMalePlayer2 + " wins the game.";

                        if(player1Serving(totalNumberOfGamesPlayed))
                        {

                            publishString = randomMalePlayer2 + " has broken " + randomMalePlayer1 + "s serve";
                            publishMessage(randomCourt, publishString);

                        }

                        getGameNumberAsText(randomMalePlayer2, randomCourt);

                        // Update the set score
                        currentSetScore[1]++;
                        totalNumberOfGamesPlayed++;

                        if(player1Serving(totalNumberOfGamesPlayed))
                        {

                            publishString = randomMalePlayer2 + " has broken " + randomMalePlayer1 + "s serve";
                            publishMessage(randomCourt, publishString);

                        }

                        numberOfGamesPlayedInThisSet = currentSetScore[0] + currentSetScore[1];
                        numberOfSetsPlayed = setsScore[0] + setsScore[1];

                        publishString = "Score in games = " + currentSetScore[0] + "-" + currentSetScore[1];
                        publishMessage(randomCourt, publishString);

                        checkSetWon(randomMalePlayer1, randomMalePlayer2, randomCourt);
                        checkMatchWon(randomMalePlayer1, randomMalePlayer2, randomCourt);

                    }

                    else if(currentGameScore[0].equals("AD"))
                    {
                        currentGameScore[0] = "40";
                        currentGameScore[1] = "40";

                        publishString = randomMalePlayer1 + ":" + currentGameScore[0] + "-" + currentGameScore[1] + ":" + randomMalePlayer2;
                        publishMessage(randomCourt, publishString);
                    }
                    else
                    {
                        currentGameScore[0] = "0";
                        currentGameScore[1] = "0";
                        System.out.println("ERROR");
                    }
                }

                break;

            }

            try
            {
                java.lang.Thread.sleep(delay*1000);
            }
            catch(java.lang.InterruptedException e)
            {
                e.printStackTrace();
            }

        }
    }

    public void insertServiceData(String machineName, String userID, String password,String randomCourt)
    {

        String[] malePlayers =
        {
            "Roger Federer",
            "Andy Roddick",
            "Guillermo Coria",
            "Juan Carlos Ferrero",
            "Carlos Moya",
            "Andre Agassi",
            "Rainer Schuerrler",
            "Tim Henman",
            "Sebastien Grosjean",
            "Nicolas Massu",
            "Leyton Hewitt",
            "Paradorn Scrichaphan",
            "Jiri Novak",
            "Sjeng Schalken",
            "Fernando Gonzales"};

        String[] femalePlayers =
        {
            "Justine Henin-Hardenne",
            "Serena Williams",
            "Amelie Mauresmo",
            "Venus Williams",
            "Lindsay Davenport",
            "Anastasia Myskina",
            "Jennifer Capriati",
            "Nadia Petrova",
            "Elena Dementieva",
            "Vera Zvonareva",
            "Svetlana Kuznetsova",
            "Ai Sugiyama",
            "Chanda Rubin",
            "Paola Suarez",
            "Silvia Farina Elia",
            "Patty Schnyder"};

        while(true)
        {
            //Drivers loaded successfully
            try
            {

                String randomMalePlayer = null;
                String randomFemalePlayer = null;
                int randomFirstServiceSpeed = 0;
                java.util.Date currentDate;
                PreparedStatement prepStat = null;

                SimpleDateFormat sdf = new SimpleDateFormat();
                sdf.applyPattern("yyyy-MM-dd-HH.mm.ss.SSS");

                // Set the variables up
                Statement query = con.createStatement();

                // First of all each Player will have his/her own topic

                int typeOfdata = (int) (4 * (Math.random()));

                switch(typeOfdata)
                {
                case 0 :
                    // PublisherType - First Service Speed Male Player
                    // Generate a number between 100 and 160         
                    randomFirstServiceSpeed = (int) (60 * (Math.random())) + 100;
                    // Get a random malePlayer out of our dataset
                    randomMalePlayer = malePlayers[((int) ((malePlayers.length) * Math.random()))];
                    publishString = "Last First Service Speed = " + randomFirstServiceSpeed + "kph";
                    publishMessage(randomCourt, publishString);

                    break;

                case 1 :
                    // PublisherType - Second Service Speed Male Player
                    // Generate a number between 100 and 160         
                    randomFirstServiceSpeed = (int) (60 * (Math.random())) + 100;
                    // Get a random malePlayer out of our dataset
                    randomMalePlayer = malePlayers[((int) ((malePlayers.length) * Math.random()))];
                    publishString = "Last Second Service Speed = " + randomFirstServiceSpeed + "kph";
                    publishMessage(randomCourt, publishString);
                    break;

                case 2 :
                    // PublisherType - First Service Speed Female Player
                    // Generate a number between 90 and 130         
                    randomFirstServiceSpeed = (int) (30 * (Math.random())) + 90;
                    // Get a random malePlayer out of our dataset
                    randomFemalePlayer = femalePlayers[((int) ((femalePlayers.length) * Math.random()))];

                    publishString = "Last First Service Speed = " + randomFirstServiceSpeed + "kph";
                    publishMessage(randomCourt, publishString);

                    break;

                case 3 :
                    // PublisherType - First Service Speed Female Player
                    // Generate a number between 90 and 130         
                    randomFirstServiceSpeed = (int) (30 * (Math.random())) + 90;
                    // Get a random malePlayer out of our dataset
                    randomFemalePlayer = femalePlayers[((int) ((femalePlayers.length) * Math.random()))];

                    publishString = "Last Second Service Speed = " + randomFirstServiceSpeed + "kph";
                    publishMessage(randomCourt, publishString);

                    break;

                }

                try
                {
                    //System.out.println("Sleeping for 1000ms before next insert.");
                    java.lang.Thread.sleep(10);
                }
                catch(java.lang.InterruptedException e)
                {
                    e.printStackTrace();
                }

            }
            catch(java.sql.SQLException ex)
            {

                System.err.println("\n--- SQLException caught ---\n");
                while(ex != null)
                {
                    System.err.println("Message:   " + ex.getMessage());
                    System.err.println("SQLState:  " + ex.getSQLState());
                    System.err.println("ErrorCode: " + ex.getErrorCode());
                    ex = ex.getNextException();
                    System.out.println("");
                }
            }
        }
    }

    private void publishMessage(String topicName, String message)
    {
        // Create our JMS objects
        try
        {
            topic = session.createTopic("Tournament/" + topicName);

            // Create TopicPublisher
            pub = session.createPublisher(topic);
        }
        catch(JMSException e)
        {
            output("Unable to create JMS objects");
            e.printStackTrace();
            Exception le = e.getLinkedException();
            if(le != null)
                le.printStackTrace();

                // Close our JMS objects
            if(pub != null)
                try
                {
                    pub.close();
                }
                catch(JMSException je)
                {
                }
            if(session != null)
                try
                {
                    session.close();
                }
                catch(JMSException je)
                {
                }
            if(conn != null)
                try
                {
                    conn.close();
                }
                catch(JMSException je)
                {
                }

            System.exit(0);
        }

        try
        {
            // Received input, create and publish message from it
            TextMessage msg = session.createTextMessage();
            msg.setText(message);
            pub.publish(msg);
            output("Message published: '" + message + "'");
        }
        catch(JMSException e)
        {
            output("Error while publishing messages");
            e.printStackTrace();
            Exception le = e.getLinkedException();
            if(le != null)
                le.printStackTrace();
        }
        catch(Exception e)
        {
            output("Error while publishing messages");
            e.printStackTrace();
        }

    }

    private void tieBreak(String randomMalePlayer1, String randomMalePlayer2, String randomCourt)
    {
        TIEBREAK : while(true)
        {

            int pointWinner = (int) (2 * (Math.random()));

            switch(pointWinner)
            {
            case 0 :
                tieBreak[0]++;

                publishString = randomMalePlayer1 + " wins the point";
                publishMessage(randomCourt, publishString);

                publishString = "Tie Break Score is " + tieBreak[0] + "-" + tieBreak[1];
                publishMessage(randomCourt, publishString);

                if((tieBreak[0] == 7))
                {

                    numberOfSetsPlayed = (setsScore[0]) + setsScore[1];

                    switch(numberOfSetsPlayed)
                    {
                    case 1 :
                        setsPlayedPrefix = "first";
                        break;
                    case 2 :
                        setsPlayedPrefix = "second";
                        break;
                    case 3 :
                        setsPlayedPrefix = "third";
                        break;
                    case 4 :
                        setsPlayedPrefix = "forth";
                        break;
                    case 5 :
                        setsPlayedPrefix = "fifth";
                        break;
                    }

                    // Reset the counters
                    currentSetScore[0] = 0;
                    currentSetScore[1] = 0;

                    // Player 1 wins the set. Update the set count
                    publishString = randomMalePlayer1 + " wins the " + setsPlayedPrefix + " set on a Tie Break";
                    publishMessage(randomCourt, publishString);

                    // Update the set score
                    setsScore[0]++;

                    publishString = "Score in sets is " + randomMalePlayer1 + ":" + setsScore[0] + "-" + setsScore[1] + ":" + randomMalePlayer2;
                    publishMessage(randomCourt, publishString);

                    break TIEBREAK;
                }
                else if(tieBreak[0] == 6)
                {

                    publishString = randomMalePlayer1 + " has " + (tieBreak[0] - tieBreak[1]) + " set points.";
                    publishMessage(randomCourt, publishString);

                }

            case 1 :
                tieBreak[1]++;

                publishString = randomMalePlayer2 + " wins the point";
                publishMessage(randomCourt, publishString);

                publishString = "Tie Break Score is " + tieBreak[0] + "-" + tieBreak[1];
                publishMessage(randomCourt, publishString);

                if((tieBreak[1] == 7))
                {

                    numberOfSetsPlayed = (setsScore[0]) + setsScore[1];

                    switch(numberOfSetsPlayed)
                    {
                    case 1 :
                        setsPlayedPrefix = "first";
                        break;
                    case 2 :
                        setsPlayedPrefix = "second";
                        break;
                    case 3 :
                        setsPlayedPrefix = "third";
                        break;
                    case 4 :
                        setsPlayedPrefix = "forth";
                        break;
                    case 5 :
                        setsPlayedPrefix = "fifth";
                        break;
                    }

                    // Reset the counters
                    currentSetScore[0] = 0;
                    currentSetScore[1] = 0;

                    // Player 1 wins the set. Update the set count
                    publishString = randomMalePlayer2 + " wins the " + setsPlayedPrefix + " set on a Tie Break";
                    publishMessage(randomCourt, publishString);

                    // Update the set score
                    setsScore[1]++;

                    publishString = "Score in sets is " + randomMalePlayer1 + ":" + setsScore[0] + "-" + setsScore[1] + ":" + randomMalePlayer2;
                    publishMessage(randomCourt, publishString);

                    break TIEBREAK;
                }
                else if(tieBreak[1] == 6)
                {

                    publishString = randomMalePlayer2 + " has " + (tieBreak[1] - tieBreak[0]) + " set points.";
                    publishMessage(randomCourt, publishString);

                }

            }
        }

    }

    // Check if the match has been won
    private void checkMatchWon(String randomMalePlayer1, String randomMalePlayer2, String randomCourt)
    {

        if(setsScore[0] == 3)
        {
            publishString = randomMalePlayer1 + " wins the match by " + setsScore[0] + " sets to " + setsScore[1];
            publishMessage(randomCourt, publishString);
            System.exit(1);
        }
        else if(setsScore[1] == 3)
        {
            publishString = randomMalePlayer2 + " wins the match by " + setsScore[1] + " sets to " + setsScore[0];
            publishMessage(randomCourt, publishString);
            System.exit(1);
        }

    }

    // Check if the set has been won
    private void checkSetWon(String randomMalePlayer1, String randomMalePlayer2, String randomCourt)
    {

        // First Check if it is now a tie Break
        if((currentSetScore[0] == 6) & (currentSetScore[1] == 6))
        {

            // Tie Break
            currentGameScore[0] = "0";
            currentGameScore[1] = "0";

            tieBreak(randomMalePlayer1, randomMalePlayer2, randomCourt);
        }

        // Now check if the set has been won
        else if(((currentSetScore[0] == 6) & (currentSetScore[1] < 5)) | (currentSetScore[0] == 7))
        {

            setsScore[0]++;

            numberOfSetsPlayed = (setsScore[0]) + setsScore[1];

            switch(numberOfSetsPlayed)
            {
            case 1 :
                setsPlayedPrefix = "first";
                break;
            case 2 :
                setsPlayedPrefix = "second";
                break;
            case 3 :
                setsPlayedPrefix = "third";
                break;
            case 4 :
                setsPlayedPrefix = "forth";
                break;
            case 5 :
                setsPlayedPrefix = "fifth";
                break;
            }

            // Player 1 wins the set. Update the set count
            publishString = randomMalePlayer1 + " wins the " + setsPlayedPrefix + " set by " + currentSetScore[0] + "-" + currentSetScore[1];
            publishMessage(randomCourt, publishString);

            publishString = "Score in sets is " + randomMalePlayer1 + ":" + setsScore[0] + "-" + setsScore[1] + ":" + randomMalePlayer2;
            publishMessage(randomCourt, publishString);

            // Reset the counters
            currentSetScore[0] = 0;
            currentSetScore[1] = 0;
        }
        else if(((currentSetScore[1] == 6) & (currentSetScore[0] < 5)) | (currentSetScore[1] == 7))
        {

            setsScore[1]++;

            numberOfSetsPlayed = (setsScore[0]) + setsScore[1];

            switch(numberOfSetsPlayed)
            {
            case 1 :
                setsPlayedPrefix = "first";
                break;
            case 2 :
                setsPlayedPrefix = "second";
                break;
            case 3 :
                setsPlayedPrefix = "third";
                break;
            case 4 :
                setsPlayedPrefix = "forth";
                break;
            case 5 :
                setsPlayedPrefix = "fifth";
                break;
            }

            // Player 1 wins the set. Update the set count
            publishString = randomMalePlayer2 + " wins the " + setsPlayedPrefix + " set by " + currentSetScore[1] + "-" + currentSetScore[0];
            publishMessage(randomCourt, publishString);

            publishString = "Score in sets is " + randomMalePlayer1 + ":" + setsScore[0] + "-" + setsScore[1] + ":" + randomMalePlayer2;
            publishMessage(randomCourt, publishString);

            // Reset the counters
            currentSetScore[0] = 0;
            currentSetScore[1] = 0;

        }
    }

    private void getGameNumberAsText(String winningPlayer, String randomCourt)
    {

        switch(numberOfGamesPlayedInThisSet)
        {
        case 1 :
            gamesPlayedPrefix = "first";
            break;
        case 2 :
            gamesPlayedPrefix = "second";
            break;
        case 3 :
            gamesPlayedPrefix = "third";
            break;
        case 4 :
            gamesPlayedPrefix = "forth";
            break;
        case 5 :
            gamesPlayedPrefix = "fifth";
            break;
        case 6 :
            gamesPlayedPrefix = "sixth";
            break;
        case 7 :
            gamesPlayedPrefix = "seventh";
            break;
        case 8 :
            gamesPlayedPrefix = "eighth";
            break;
        case 9 :
            gamesPlayedPrefix = "ninth";
            break;
        case 10 :
            gamesPlayedPrefix = "tenth";
            break;
        case 11 :
            gamesPlayedPrefix = "eleventh";
            break;
        case 12 :
            gamesPlayedPrefix = "twelveth";
            break;
        default :
            System.out.println("Error in getGameNumberAsText : " + numberOfGamesPlayedInThisSet);
        }

        switch(numberOfSetsPlayed)
        {
        case 0 :
            setsPlayedPrefix = "first";
            break;
        case 1 :
            setsPlayedPrefix = "second";
            break;
        case 2 :
            setsPlayedPrefix = "third";
            break;
        case 3 :
            setsPlayedPrefix = "forth";
            break;
        case 4 :
            setsPlayedPrefix = "final";
            break;
        default :
            System.out.println("Error in getGameNumberAsText");

        }

        publishString = winningPlayer + " wins the " + gamesPlayedPrefix + " game in the " + setsPlayedPrefix + " set.";
        publishMessage(randomCourt, publishString);

    }

    private void whosServing(String randomMalePlayer1, String randomMalePlayer2, String randomCourt)
    {

        int firstOrSecondServe = 0;
        int firstServeSpeed = 0;
        int secondServeSpeed = 0;

        if(player1Serving(totalNumberOfGamesPlayed))
        {
            publishString = randomMalePlayer1 + " serves";
            publishMessage(randomCourt, publishString);

            // If this player was serving, generate a service speed
            if(player1Serving(totalNumberOfGamesPlayed))
            {
                // 50/50 chance that this was a first or second serve
                firstOrSecondServe = (int) (2 * (Math.random()));

                switch(firstOrSecondServe)
                {
                case 0 :
                    // First Serve went in
                    firstServeSpeed = (int) (60 * (Math.random())) + 100;
                    publishString = randomMalePlayer1 + " first serve speed = " + firstServeSpeed + "kph";
                    publishMessage(randomCourt, publishString);
                    p1_numberOfFirstServesIn++;
                    break;
                case 1 :
                    // Second Serve went in
                    secondServeSpeed = (int) (60 * (Math.random())) + 80;
                    publishString = randomMalePlayer1 + " second serve speed = " + secondServeSpeed + "kph";
                    publishMessage(randomCourt, publishString);
                    p1_numberOfSecondServesIn++;
                    break;
                }
            }

        }
        else
        {
            publishString = randomMalePlayer2 + " serves";
            publishMessage(randomCourt, publishString);

            // 50/50 chance that this was a first or second serve
            firstOrSecondServe = (int) (2 * (Math.random()));

            switch(firstOrSecondServe)
            {
            case 0 :
                // First Serve went in
                firstServeSpeed = (int) (60 * (Math.random())) + 100;
                publishString = randomMalePlayer2 + " first serve speed = " + firstServeSpeed + "kph";
                publishMessage(randomCourt, publishString);
                p2_numberOfFirstServesIn++;
                break;
            case 1 :
                // Second Serve went in
                secondServeSpeed = (int) (60 * (Math.random())) + 80;
                publishString = randomMalePlayer2 + " second serve speed = " + secondServeSpeed + "kph";
                publishMessage(randomCourt, publishString);
                p2_numberOfSecondServesIn++;
                break;
            }
        }

        return;
    }

    private void isBreakPoint(String randomMalePlayer1, String randomMalePlayer2, String randomCourt)
    {

        if(player1Serving(totalNumberOfGamesPlayed))
        {

            // Player 1 is serving. Check to see if the other guy is on 40

            if((currentGameScore[0].equals("0")) & (currentGameScore[1].equals("40")))
            {

                publishString = randomMalePlayer2 + " has 3 break points";
                publishMessage(randomCourt, publishString);
            }
            else if((currentGameScore[0].equals("15")) & (currentGameScore[1].equals("40")))
            {
                publishString = randomMalePlayer2 + " has 2 break points";
                publishMessage(randomCourt, publishString);

            }
            else if((currentGameScore[0].equals("30")) & (currentGameScore[1].equals("40")))
            {
                publishString = randomMalePlayer2 + " has 1 break point";
                publishMessage(randomCourt, publishString);

            }
            else if((currentGameScore[0].equals("40")) & (currentGameScore[1].equals("AD")))
            {
                publishString = randomMalePlayer2 + " has 1 break point";
                publishMessage(randomCourt, publishString);

            }

        }
        else
        {

            if((currentGameScore[1].equals("0")) & (currentGameScore[0].equals("40")))
            {
                publishString = randomMalePlayer1 + " has 3 break points";
                publishMessage(randomCourt, publishString);
            }
            else if((currentGameScore[1].equals("15")) & (currentGameScore[0].equals("40")))
            {
                publishString = randomMalePlayer1 + " has 2 break points";
                publishMessage(randomCourt, publishString);

            }
            else if((currentGameScore[1].equals("30")) & (currentGameScore[0].equals("40")))
            {
                publishString = randomMalePlayer1 + " has 1 break point";
                publishMessage(randomCourt, publishString);

            }
            else if((currentGameScore[1].equals("40")) & (currentGameScore[0].equals("AD")))
            {
                publishString = randomMalePlayer1 + " has 1 break point";
                publishMessage(randomCourt, publishString);

            }
        }

    }

    private boolean player1Serving(int numberOfGamesPlayed)
    {
        // Determine who is currently serving.
        //
        // In the sim. player1 vs player2
        //
        // Player1 serves first. Hence he will be serving in the 1st,3rd,5th,7th game
        // Player 2 will be serving in the 2nd,4th,6th etc.
        //
        if((numberOfGamesPlayed % 2) > 0)
        {
            // Odd number - player1 is serving
            return true;
        }
        else
        {
            // Even number - player2 is serving
            return false;
        }
    }

    private static void output(String s)
    {
        System.out.println(s);
    }

    private static void usage()
    {
        System.out.println("\n\n-------------------------------------------------------------------------------------------------");
        System.out.println("\tReplay Tennis Message Simulator");
        System.out.println("\n\tUsage\n");
        System.out.println("\t\tjava samples.TennisPublisherSimulator <TopicConnectionFactory> [jndiICF] [jndiURL] <DelayBetweenPoints>\n");
        System.out.println("\tExample usage\n");
        System.out.println("\t\tjava samples.TennisPublisherSimulator IP_2004 5");
        System.out.println("\t\tjava samples.TennisPublisherSimulator IP_2004 com.sun.jndi.fscontext.RefFSContextFactory file:/c:/JNDI 5");
        System.out.println("\n-------------------------------------------------------------------------------------------------");
        System.exit(0);
    }

    public static void main(String args[])
    {
        try
        {
            if(args.length == 0)
            {
                usage();
            }
            else
            {
                if(args.length == 2)
                {
                    delay = Integer.parseInt(args[1]);
                    output("Set jndiICF to "+icf);
                    output("Set jndiURL to "+url);
                    output("Set delay to "+delay);
                }
                else
                {
                    icf=args[1];
                    output("Set jndiICF to "+icf);
                    url=args[2];
                    output("Set jndiURL to "+url);
                    delay = Integer.parseInt(args[3]);
                    output("Set delay to "+delay);
                }

            }

            TennisPublisherSimulator play = new TennisPublisherSimulator();
            play.insertGameSimulation(args);
            try
            {
                conn.close();
            }
            catch(JMSException e)
            {
                e.printStackTrace();
            }
        }
        catch(Exception e)
        {
            output("Error with command line parameters");
            usage();
        }

    }

}


